"""
for this not sure what I want, I think since I am just using SHAP as a >0 for 3/10 models, I will think about this and
plot it when I know what I want, also gain plots could be useful but this is super not interesting and not the main
point so it doesnt have to be perfect

"""
import matplotlib.pyplot as plt
from tqdm.auto import tqdm

from whacc import utils
import numpy as np
from whacc import image_tools
from natsort import natsorted
import pickle
import pandas as pd
import os
import copy
import seaborn as sns
from keras.preprocessing.image import ImageDataGenerator
import h5py
from imgaug import augmenters as iaa  # optional program to further augment data

import lightgbm as lgb

import shap


def load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0, label_key='labels'):
    if isinstance(h5_in, list):
        all_x = [];
        all_y = []
        for k in h5_in:
            tmp_x, tmp_y = load_data(k, feature_list, split_int_n_chunks=split_int_n_chunks,
                                     split_chunk_ind=split_chunk_ind, label_key=label_key)
            all_x.append(tmp_x)
            all_y.append(tmp_y)
            del tmp_x, tmp_y

        all_x = np.vstack(all_x)
        all_y = np.hstack(all_y)
        return all_x, all_y
    all_x = None
    for k in feature_list:
        if all_x is None:
            all_x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype(
                'float32')
        else:
            x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype('float32')
            if len(x.shape) > 1:
                all_x = np.hstack((all_x, x))
            else:
                all_x = np.hstack((all_x, x[:, None]))

    all_y = image_tools.get_h5_key_and_concatenate(h5_in, label_key)[split_chunk_ind::split_int_n_chunks].astype(
        'float32')
    return all_x, all_y


def rm_nan_rows(a, a2=None):
    indx = ~np.isnan(a).any(axis=1)
    if a2 is None:
        return a[indx, :]
    else:
        return a[indx, :], a2[indx]


def get_feature_data_names(feature_list, n=2048):
    featuredata_names = [k.replace('FD__', '').replace('____', '') for k in feature_list]
    final_feature_names = []
    names = []
    nums = []
    for i, (k1, k2) in enumerate(zip(featuredata_names, feature_list)):
        if '_TOTAL_' in k2:
            names.append(k1)
            nums.append(np.arange(1))
            final_feature_names.append(k1)
        else:

            names.append(np.repeat(k1, n))
            nums.append(np.arange(n))
            final_feature_names.append([i1 + '_' + str(i2) for i1, i2 in zip(names[-1], nums[-1])])
    out = [np.hstack(k) for k in [final_feature_names, names, nums, featuredata_names]]
    out[0] = list(out[0])
    out[-1] = list(out[-1])
    return out


h5_in = '/Users/phil/Desktop/LIGHT_GBM/reg_80_border_and_andrew_both_samson_both__test_num_0_percent_15.h5'
mod_fn = '/Users/phil/Desktop/LIGHT_GBM/model_saves/model_num_0_of_10.pkl'

with open(mod_fn, 'rb') as f:
    model = pickle.load(f)

explainer = shap.TreeExplainer(model)

feature_list = natsorted(utils.lister_it(utils.print_h5_keys(h5_in, 1, 0), keep_strings='FD__'))
final_feature_names, feature_names, feature_nums, feature_list_short = get_feature_data_names(feature_list)
x_train, y_train = load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0)

shap_values = explainer.shap_values(x_train)
shap_values_1 = shap_values[1]

shap.summary_plot(shap_values_1, x_train, feature_names=final_feature_names, max_display=20)

shap_feature_mean = np.nanmean(np.abs(shap_values_1), axis=0)

_, bi = utils.lister_it(final_feature_names, 'FD_TOTAL', return_bool_index=True)
shap_feature_mean_TOTAL = shap_feature_mean[bi]
shap_feature_mean_other = shap_feature_mean[bi == 0]
final_feature_names_other = np.asarray(final_feature_names)[bi == 0]

num_neurons = 2048
neurons_mean = []
for k in range(num_neurons):
    neurons_mean.append(np.nanmean(shap_feature_mean_other[k::num_neurons]))
feature_mean = []
for k in range(len(shap_feature_mean_other) // num_neurons):
    feature_mean.append(np.nanmean(shap_feature_mean_other[k * num_neurons: k * num_neurons + num_neurons]))

max_bin = 1.0001 * np.max(feature_mean + neurons_mean + list(shap_feature_mean_TOTAL))

bins = np.linspace(1 * 10 ** -12, max_bin, 100)
label_names = ['Feature', 'Neurons', 'TOTAL feature']
tmp1 = plt.hist([feature_mean, neurons_mean, shap_feature_mean_TOTAL], bins,
                label=label_names)
list_of_each = [feature_mean, neurons_mean, list(shap_feature_mean_TOTAL)]
group_name = np.concatenate([[name] * len(k) for name, k in zip(label_names, list_of_each)])
df = pd.DataFrame(zip(np.concatenate(list_of_each), group_name), columns=['mean value', 'group'])

tmp1 = df.groupby('group')
tmp1.plot.kde()
tmp1 = df.sort_values('mean value')
tmp1.plot()

for k, name in zip(list_of_each, label_names):
    x = np.arange(len(k))
    x = x / x[-1]
    plt.plot(sorted(k), x, '.', label=name)
plt.legend()

print(np.round(100 * np.mean(shap_feature_mean > 0), 2), '%')

num_neurons = 2048
neurons_not_0 = []
for k in range(num_neurons):
    neurons_not_0.append(np.sum(shap_feature_mean_other[k::num_neurons] > 0))
feature_not_0 = []
num_transforms = len(shap_feature_mean_other) // num_neurons
for k in range(num_transforms):
    feature_not_0.append(np.sum(shap_feature_mean_other[k * num_neurons: k * num_neurons + num_neurons] > 0))

bins = np.linspace(0, .45, 45)
plt.hist([np.asarray(neurons_not_0) / num_transforms], bins=bins)
# plt.xlim([0, .42])
plt.figure()
plt.hist([np.asarray(feature_not_0) / num_neurons], bins=bins)
# plt.xlim([0, .42])


# final_feature_names, feature_names, feature_nums, feature_list_short
np.argmax(np.asarray(neurons_not_0) / num_transforms)

tmp1 = np.flip(np.argsort(np.asarray(feature_not_0) / num_neurons))

x = np.asarray(feature_not_0) / num_neurons
for k in tmp1:
    tmp2.append()
    print(feature_list_short[k])
    print(feature_not_0[k])







h5_in = '/Users/phil/Desktop/LIGHT_GBM/reg_80_border_and_andrew_both_samson_both__test_num_0_percent_15.h5'




for n in feature_list:
    x = n.split('____')[0][-3:]
    i = np.where([k in '-0123456789' for k in x])[0][0]
    print(x[i:])
    # print(np.asarray(x)[[k in '-0123456789' for k in x]])
    # asdf




#
# with h5py.File(h5_in, 'r') as h:
#    for k in h.keys():
#        if 'FD__' in k:
#            print(k)
h5_final_keys_sorted = utils.print_h5_keys(h5_in, 1, 0)




feature_list == natsorted(utils.lister_it(utils.print_h5_keys(h5_in, 1, 0), keep_strings='FD__')) # I do use os sorted here
final_feature_names, feature_names, feature_nums, feature_list_short = get_feature_data_names(feature_list)
# directories with data and models
bd = '/Volumes/rig1 EPHUS pc backup /LIGHT_GBM/FEATURE_DATA'
shap_contribution_list = []
shap_len_samples_list = []
fn_list = []
mod_list = []
all_models = utils.get_files('/Users/phil/Desktop/LIGHT_GBM/model_saves/', '*.pkl')
for mod_fn in tqdm(all_models):
    with open(mod_fn, 'rb') as f:
        model = pickle.load(f)
    # load model
    explainer = shap.TreeExplainer(model)
    for k in tqdm(model.info_dict['val_data_files']):
        mod_list.append(mod_fn.split('/')[-1].split('_of')[0])
        # load data
        k = k.split('\\')[-1].split('/')[-1]
        h5_in = bd + os.sep + k
        fn_list.append(h5_in)
        x_train, y_train = load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0)
        shap_values = explainer.shap_values(x_train)
        shap_values_1 = shap_values[1]
        shap_contribution_list.append(np.nanmean(np.abs(shap_values_1), axis=0))
        shap_len_samples_list.append(shap_values_1.shape[0])

df = pd.DataFrame(zip(shap_contribution_list, shap_len_samples_list, fn_list, mod_list), columns= ['shap', 'len_x', 'file name', 'mod name'])
df['']
utils.save_obj(df, '/Users/phil/Desktop/all_model_shap_dataframe')
tmp1 = df.groupby(['mod name'])
tmp1['shap'].mean()

tmp1 = []
tmp2 = np.asarray(shap_contribution_list)
for mn in np.unique(mod_list):
    tmp1.append(np.nanmean(tmp2[np.asarray(mod_list)==mn, :], axis = 0))

tmp1 = np.asarray(tmp1)
print(tmp1.shape)

tmp5 = np.sum(tmp1>0, axis=0)

plt.imshow(tmp1[:, :100])

tmp3 = []
for k in tmp1:
    tmp3.append(np.where(k>0)[0])
tmp3 = np.concatenate(tmp3)

out_0 = plt.hist(tmp3, bins = np.arange(-.5, 84009+.5, 1))

features_out_of_10 = copy.deepcopy(out_0[0])

out_1 = plt.hist(tmp3, bins = np.arange(-.5, 84009+.5, 2048))

out_2 = plt.hist(tmp3, bins = np.arange(-.5, 84009+.5, 2048))



for k in range(10):
    print(np.round(100 * np.mean(features_out_of_10 > k), 2), '% have ', k+1, ' or more')
    print('count ', np.sum(features_out_of_10 > k))



np.save('/Users/phil/Desktop/LIGHT_GBM/features_out_of_10', features_out_of_10)

greater_than_or_equal_to = 10
np.sum(features_out_of_10 >= greater_than_or_equal_to)
best_features = np.where(features_out_of_10[:-41] >= greater_than_or_equal_to)[0]
best_TOTAL_features = len(features_out_of_10[:-41])+np.where(features_out_of_10[-41:] >= greater_than_or_equal_to)[0]

print(best_features//2048)
print(sorted(best_features%2048))

print(np.unique(best_features//2048))
print(np.unique(best_features%2048))

print(len(np.unique(best_features//2048)))
print(len(np.unique(best_features%2048)))


feature_inds = np.unique(best_features//2048)
neuron_inds = np.unique(best_features%2048)

keep_features_index = np.repeat(neuron_inds, len(feature_inds)).reshape([len(neuron_inds), len(feature_inds)])
keep_features_index = keep_features_index + feature_inds*2048 - 2048
keep_features_index = np.append(keep_features_index.flatten(),  best_TOTAL_features)
# final_feature_names, feature_names, feature_nums, feature_list_short

tmp1 = [features_out_of_10[k]for k in keep_features_index]


for k in np.unique(tmp1//2048):
    print(feature_list_short[k])


final_feature_names_TOTAL, bi = utils.lister_it(final_feature_names, 'FD_TOTAL', return_bool_index=True)
features_out_of_10_TOTAL = features_out_of_10[bi]
features_out_of_10_other = features_out_of_10[bi == 0]
final_feature_names_other = np.asarray(final_feature_names)[bi == 0]


num_neurons = 2048
neurons_not_0 = []
for k in range(num_neurons):
    neurons_not_0.append(np.sum(features_out_of_10_other[k::num_neurons] > 0))
feature_not_0 = []
num_transforms = len(features_out_of_10_other) // num_neurons
for k in range(num_transforms):
    feature_not_0.append(np.sum(features_out_of_10_other[k * num_neurons: k * num_neurons + num_neurons] > 0))


# bins = np.linspace(0, .45, 45)

neurons_not_0_tmp = sorted(np.asarray(neurons_not_0)/num_transforms)
plt.bar( range(len(neurons_not_0_tmp)), neurons_not_0_tmp)
plt.grid(b=True, which='both')
plt.minorticks_on()
plt.ylim([0, .92])

plt.figure()
feature_not_0_tmp = sorted(np.asarray(feature_not_0)/num_neurons)
plt.bar( range(len(feature_not_0_tmp)), feature_not_0_tmp)
plt.grid(b=True, which='both')
plt.minorticks_on()
plt.ylim([0, .92])

plt.figure()
plt.bar( range(len(features_out_of_10_TOTAL)), features_out_of_10_TOTAL/10)
plt.grid(b=True, which='both')
plt.minorticks_on()
plt.ylim([0, .92])






mean_50 = []
for k in range(1+len(neurons_not_0_tmp)//50):
    mean_50.append(np.mean(neurons_not_0_tmp[k*50:k*50+50]))
plt.figure()
plt.bar(np.arange(len(mean_50)),mean_50)
plt.grid(b=True, which='both')
plt.minorticks_on()
plt.ylim([0, .92])




plt.figure()
plt.plot(range(len(feature_not_0_tmp)), feature_not_0_tmp, '.k', label='augmented features')
plt.plot(range(len(feature_not_0_tmp)), mean_50, '.r', label='neurons')
plt.grid(b=True, which='both')
plt.minorticks_on()
plt.ylim([0, .92])
plt.legend()

# plt.bar(range(len(feature_not_0_tmp)), [feature_not_0_tmp, mean_50])
#
# plt.bar([range(len(feature_not_0_tmp)), range(len(feature_not_0_tmp))], [feature_not_0_tmp, mean_50])

# plt.grid(b=True, which='both')
# plt.minorticks_on()
# plt.ylim([0, .92])




# tmp1 = np.asarray(shap_contribution_list)
# for k in range(tmp1.shape[0]//2):
#     np.mean()
#

# loop through zipped models and h5 (val/test pairs load together)

# get full shap values

# get abs and mean shap values of index [1] (contributions to touch)
"""
I guess I could just go with all the features that overlap for each model
then just run the model with those features (selecting using indexing from the h5) 
show that they are similar 
then choose those and train with the full dataset (maybe even augmented)
then show that they can work with unseen data 
then show they can work with transfer learning 


one test I can do is train on different looking images one model for each session, to see if the overlap is the same 
or if there is some selection for each 
if there is some selection for each then TL wont work as well if I only select the top ~8% x neurons 
so like if there is a set that like this 10% and another set that likes 8% and half the neurons overlap but I choose the 
10% neurons then only 5% of those will be useful for predicitng making the model not TL well 


which group has a lot of clear 0's and then clear winners to choose from how should I choose features
do the plots 
separate by neuron take mean for each plot red hist
separate by feature (not total features) type take mean for each plot blue hist 
separate by total features take mean for each plot green hist 

plot sorted of these groups 

choose the ones we want, test how good the model is


"""
# df_x = pd.DataFrame(x_train, columns=final_feature_names)
# for name in df_x.columns:
#     shap.dependence_plot(name, shap_values[1], df_x, display_features=df_x)
#     asdfasdfsdfasdf

#
#
# """
# shap.summary_plot
# need to figure out how this work or just get the shap values and make predicitons and shap values again and use the
# shap package otherwise IDK if they jsut take the mean or whatever
# """
#
#
#
# np_in = '/Users/phil/Desktop/LIGHT_GBM/SHAP_VAL_num_0.npy'
# shap_vals = np.load(np_in, mmap_mode='r')
#
# _, bi_2048_set = utils.lister_it(final_feature_names, remove_string=['FD_TOTAL'], return_bool_index=True)
# feature_nums_set = feature_nums[bi_2048_set]
#
# shap_neurons = dict = {}
# for k in np.unique(feature_nums_set):
#   shap_neurons[str(k)]
#   shap_vals[feature_nums_set==k]
#
#
#
# _, bi_2048_set = utils.lister_it(final_feature_names, remove_string=['FD_TOTAL'], return_bool_index=True)
#
#
# # shap_vals.shape  # (23435, 84010)
# tmp_shape = shap_vals[:100, :]
# """$$$$%%%%$$$$$%%%%$$$$%%%%$$$$$%%%%$$$$%%%%$$$$$%%%%$$$$%%%%$$$$$%%%%$$$$%%%%$$$$$%%%%$$$$%%%%$$$$$%%%%$$$$%%%%$$$$$%%%%"""
#
#
#
# # fd_keys = utils.print_h5_keys(h5_in, 1, 0)
# # fd_keys = utils.lister_it(fd_keys, 'FD__')
# # fd_k_normal = utils.lister_it(fd_keys, remove_string='FD_TOTAL')
# # fd_k_total = utils.lister_it(fd_keys, keep_strings='FD_TOTAL')
#
#
#
#
#
# def load_SHAP_data(np_in, feature_list, max_size=100):
#     if isinstance(h5_in, list):
#         all_x = [];
#         all_y = []
#         for k in h5_in:
#             tmp_x, tmp_y = load_data(k, feature_list, split_int_n_chunks=split_int_n_chunks,
#                                      split_chunk_ind=split_chunk_ind, label_key=label_key)
#             all_x.append(tmp_x)
#             all_y.append(tmp_y)
#             del tmp_x, tmp_y
#
#         all_x = np.vstack(all_x)
#         all_y = np.hstack(all_y)
#         return all_x, all_y
#     all_x = None
#     for k in feature_list:
#         if all_x is None:
#             all_x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype(
#                 'float32')
#         else:
#             x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype('float32')
#             if len(x.shape) > 1:
#                 all_x = np.hstack((all_x, x))
#             else:
#                 all_x = np.hstack((all_x, x[:, None]))
#
#     all_y = image_tools.get_h5_key_and_concatenate(h5_in, label_key)[split_chunk_ind::split_int_n_chunks].astype(
#         'float32')
#     return all_x, all_y
#
#
# def load_data(h5_in, feature_list, split_int_n_chunks=1, split_chunk_ind=0, label_key='labels'):
#     if isinstance(h5_in, list):
#         all_x = [];
#         all_y = []
#         for k in h5_in:
#             tmp_x, tmp_y = load_data(k, feature_list, split_int_n_chunks=split_int_n_chunks,
#                                      split_chunk_ind=split_chunk_ind, label_key=label_key)
#             all_x.append(tmp_x)
#             all_y.append(tmp_y)
#             del tmp_x, tmp_y
#
#         all_x = np.vstack(all_x)
#         all_y = np.hstack(all_y)
#         return all_x, all_y
#     all_x = None
#     for k in feature_list:
#         if all_x is None:
#             all_x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype(
#                 'float32')
#         else:
#             x = image_tools.get_h5_key_and_concatenate(h5_in, k)[split_chunk_ind::split_int_n_chunks].astype('float32')
#             if len(x.shape) > 1:
#                 all_x = np.hstack((all_x, x))
#             else:
#                 all_x = np.hstack((all_x, x[:, None]))
#
#     all_y = image_tools.get_h5_key_and_concatenate(h5_in, label_key)[split_chunk_ind::split_int_n_chunks].astype(
#         'float32')
#     return all_x, all_y



import tensorflow as tf
import matplotlib.pyplot as plt
import numpy as np
import h5py
from whacc import image_tools
from whacc import utils
import copy
from matplotlib import cm
from tqdm.autonotebook import tqdm
import pandas as pd
# from tensorflow import keras
# import time
# import os
# import pdb
def plot_3d_loc_time_confidence_pole_tracker(h5, index =None, center_xy=False, center_start_end = [0,50]):
    locations_x_y = image_tools.get_h5_key_and_concatenate(h5, 'locations_x_y')
    frame_nums = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
    max_val_stack = image_tools.get_h5_key_and_concatenate(h5, 'max_val_stack')


    X = locations_x_y[:, 0]
    Y = locations_x_y[:, 1]
    Z = np.arange(len(X))
    C = max_val_stack
    if center_xy:
        for i1, i2 in utils.loop_segments(frame_nums):

            X[i1:i2] = X[i1:i2]-np.mean(X[i1:i2][center_start_end[0]:center_start_end[1]])
            Y[i1:i2] = Y[i1:i2]-np.mean(Y[i1:i2][center_start_end[0]:center_start_end[1]])

    if index is not None:
        X = X[index]
        Y = Y[index]
        Z = Z[index]
        C = C[index]





    # C = np.array([[255, 0, 0], [0, 255, 0], [0, 0, 255]])
    fig = plt.figure()
    # ax = fig.add_subplot(111, projection = '3d')
    ax = fig.add_subplot(111, projection = '3d')
    # ax.scatter(X, Y, Z, c = C/255.0)
    ax.scatter(X, Y, Z, c = C)
    plt.show()

def thresholded_error_types(real, yhat_proba, edge_threshold=4, frame_num_array=None, thresholds=None):
    """
    Parameters
    ----------
    frame_num_array :
    real : binary actaul touch labels
    yhat_proba : probability of touch array
    edge_threshold : determines 'unacceptable' edge error length default = 4, so append and deducts greater than or
    equal to 4 will be put in a separate list and considered just as bad as touch count errors
    thresholds :

    Returns
    -------
    df --> data frame with count of errors for each error type for each threshold

    """
    if thresholds is None:
        thresholds = np.linspace(0.001, 1 - .001, 100)
    keys = ['ghost', 'miss', 'join', 'split']
    keys = keys + [k.replace('###', str(edge_threshold)) for k in ['append_###+', 'deduct_###+']] + ['append', 'deduct']
    d = dict()
    for k in keys:
        d[k] = []
    for k in thresholds:
        pred = yhat_proba > k
        a = error_analysis(real, pred, frame_num_array)
        for kk in keys[:-4]:
            d[kk].append(np.sum(np.asarray(a.all_error_type_sorted) == np.asarray([kk])))
        for kk2, kk1 in zip(keys[-4:-2], keys[-2:]):
            inds = np.where(np.asarray(a.all_error_type_sorted) == np.asarray([kk1]))[0]
            lens = np.asarray(a.error_lengths_sorted)[inds]
            d[kk1].append(np.sum(lens < edge_threshold))
            d[kk2].append(np.sum(lens >= edge_threshold))
    df = pd.DataFrame(d)
    return df


class basic_metrics():
    def __init__(self, y, yhat_float, num_thresh=50):
        self.thresh = np.linspace(0, 1, num=num_thresh)
        self.y = y
        self.yhat = yhat_float
        self.TN = []
        self.FP = []
        self.FN = []
        self.TP = []
        self.Sensitivity = []
        self.Specificity = []
        self.Precision = []
        self.G_Mean = []
        self.Accuracy = []

        for k in self.thresh:
            yhat = (self.yhat > k) * 1
            cm = np.asarray(tf.math.confusion_matrix(self.y, yhat) / len(self.y)).flatten()
            self.TN.append(cm[0])
            self.FP.append(cm[1])
            self.FN.append(cm[2])
            self.TP.append(cm[3])
            self.Sensitivity.append(self.TP[-1] / (self.TP[-1] + self.FN[-1]))
            self.Specificity.append(self.TN[-1] / (self.TN[-1] + self.FP[-1]))
            self.Precision.append(self.TP[-1] / (self.TP[-1] + self.FP[-1]))
            self.Accuracy.append((self.TP[-1] + self.TN[-1]) / (self.TP[-1] + self.TN[-1] + self.FP[-1] + self.FN[-1]))
            self.G_Mean.append(np.sqrt(self.Sensitivity[-1] * self.Specificity[-1]))

    def best_geo_mean_ind(self):
        max_inds = np.where(self.G_Mean == np.max(self.G_Mean))[0]  # get inds of max g mean
        if len(max_inds) == 1:
            return max_inds[0]
        print(11111)
        groups, _ = utils.group_consecutives(max_inds)  # group those continious segments
        group_lens = [len(k) for k in groups]  # get the lengths of each segment
        max_len_group_ind = np.argmax(
            [group_lens])  # find the max length of those (default to the first one if there are more than one)
        ind_tmp = np.floor(group_lens[max_len_group_ind] / 2).astype('int')  # find teh middle point in that segment
        best_geo_mean_ind = groups[max_len_group_ind][
            ind_tmp]  # use the inds to get the middle number of the largest segment
        return best_geo_mean_ind


# def find_closest(real, predicted, axis=0):
#     """
#     helper function of "get_error_arrays" takes in the indices of a bool list and finds the closest matching values to
#     both the right and left in separate arrays
#     Parameters
#     ----------
#     real : inds of bool list
#     predicted : inds of bool list
#     axis : int
#
#     Returns
#     -------
#
#     """
#     c = real[:, None] - predicted
#     closest_left = np.max(np.ma.masked_array(c, mask=[c > 0]), axis=axis)
#     closest_left.fill_value = 999999
#     closest_right = np.min(np.ma.masked_array(c, mask=[c < 0]), axis=axis)
#     closest_right.fill_value = -999999
#     return closest_left, closest_right
#
#
# def get_error_arrays(real_bool, predicted_bool):
#     """
#     used to get 2 matrices, the first is 2 columns of length sum((real_bool==1)*1) the left column is the distance to
#     the closest 1 TO THE LEFT in predicted_bool for each 1 in real_bool, the right column is the closest to the right.
#     the second matrix is the same but for 0 instead of one, columns for this matrix are of length sum((real_bool==1)*0)
#     predicted bool
#     Parameters
#     ----------
#     real_bool :
#     predicted_bool :
#
#     Returns
#     -------
#     """
#     real = np.where(real_bool == 1)[0]
#     predicted = np.where(predicted_bool == 1)[0]
#     left1, right1 = find_closest(real, predicted)
#
#     real = np.where(real_bool == 0)[0]
#     predicted = np.where(predicted_bool == 0)[0]
#     left0, right0 = find_closest(real, predicted)
#
#     L1 = left1.data.astype('float64')
#     L1[L1 == 999999] = np.nan
#     R1 = right1.data.astype('float64')
#     R1[R1 == -999999] = np.nan
#     L0 = left0.data.astype('float64')
#     L0[L0 == 999999] = np.nan
#     R0 = right0.data.astype('float64')
#     R0[R0 == -999999] = np.nan
#     # pdb.set_trace()
#     out_1 = np.vstack((L1, R1))
#     out_0 = np.vstack((L0, R0))
#     print('inds are L1, R1 and  L0, R0')
#     return np.transpose(out_1), np.transpose(out_0)


class pole_plot():
    def __init__(self, img_h5_file, pred_val=None, true_val=None, threshold=0.5,
                 len_plot=10, current_frame=0, figsize=[10, 5], label_nums=None, label_num_names=None, shift_by=0):
        """
        Examples
        ________
        a = analysis.pole_plot(
            '/Users/phil/Dropbox/HIRES_LAB/GitHub/Phillip_AC/autoCuratorDiverseDataset/AH0000x000000/master_train_test1.h5',
            pred_val = [0,0,0,0,0,0,0,.2,.4,.5,.6,.7,.8,.8,.6,.4,.2,.1,0,0,0,0],
            true_val = [0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0],
            len_plot = 10)

        a.plot_it()
        """
        if true_val is None:
            if 'labels' in utils.print_h5_keys(img_h5_file, return_list=True, do_print=False):
                true_val = image_tools.get_h5_key_and_concatenate([img_h5_file])
        # tmp_loc = locals()
        self.error_group_inds = None
        # if 'true_val' in tmp_loc.keys() and 'pred_val' in tmp_loc.keys():
        if pred_val is not None and true_val is not None:
            error_group_inds = np.where(true_val - pred_val != 0)[0]
            error_group_inds, _ = utils.group_consecutives(error_group_inds)
            self.error_group_inds = error_group_inds

        self.isnotebook = utils.isnotebook()
        self.img_h5_file = img_h5_file
        self.pred_val = np.asarray(pred_val)
        self.true_val = np.asarray(true_val)
        self.threshold = threshold
        self.len_plot = len_plot
        self.current_frame = current_frame
        self.figsize = figsize
        self.fig_created = False
        if pred_val is None:
            tmp3 = self.true_val
        elif true_val is None:
            tmp3 = self.pred_val
        else:
            tmp3 = np.unique(np.concatenate((self.pred_val, self.true_val)))
        self.range_labels = np.arange(np.nanmin(tmp3), np.nanmax(tmp3) + 1)
        self.ylims = [np.nanmin(tmp3) - .5, np.nanmax(tmp3) + .5]

        self.label_nums = label_nums
        self.label_num_names = label_num_names
        self.shift_by = shift_by
        self.xlims = [-.5, self.len_plot - .5]
        try:
            self.pred_val_bool = (1 * (self.pred_val > threshold)).flatten()
        except:
            self.pred_val_bool = np.asarray(None)

    def plot_it(self):

        if self.fig_created is False or self.isnotebook:  # we need to create a new fig every time if we are in colab or jupyter
            self.fig, self.axs = plt.subplots(2, figsize=self.figsize, gridspec_kw={'height_ratios': [1, 1]})
            self.fig_created = True
            # plt.subplots_adjust(hspace = .001)

        ax1 = self.axs[1]
        box = ax1.get_position()
        box.y0 = box.y0 + self.shift_by
        box.y1 = box.y1 + self.shift_by
        ax1.set_position(box)

        self.axs[0].clear()
        self.axs[1].clear()
        self.fig.suptitle('Touch prediction')
        s1 = self.current_frame
        s2 = self.current_frame + self.len_plot
        self.xticks = np.arange(0, self.len_plot)
        # plt.axis('off')
        with h5py.File(self.img_h5_file, 'r') as h:
            self.current_imgs = image_tools.img_unstacker(h['images'][s1:s2], s2 - s1)
            # plt.imshow(self.current_imgs)
            self.axs[0].imshow(self.current_imgs)#, interpolation='nearest', aspect='auto')
            self.axs[0].axis('off')

        leg = []
        # axs[1].plot([None])
        if len(self.pred_val.shape) != 0:
            plt.plot(self.pred_val[s1:s2].flatten(), color='black', marker='*', linestyle='None')
            leg.append('pred')
        # if len(self.pred_val_bool.shape) != 0:
        #     plt.plot(self.pred_val_bool[s1:s2].flatten(), '.g', markersize=10)
        #     leg.append('bool_pred')
        if len(self.true_val.shape) != 0:
            tmp1 = self.true_val[s1:s2].flatten()
            plt.scatter(range(len(tmp1)), tmp1, s=80, facecolors='none', edgecolors='r')
            leg.append('actual')
        if leg:
            plt.legend(leg, bbox_to_anchor=(1.04, 1), borderaxespad=0)
            plt.ylim(self.ylims)
            plt.xlim(self.xlims)
            _ = plt.xticks(ticks=self.xticks)
        if self.label_nums is not None and self.label_num_names is not None:
            plt.yticks(ticks=self.label_nums, labels=self.label_num_names)
        plt.grid(axis='y')

    def next(self):
        self.current_frame = self.current_frame + self.len_plot
        self.plot_it()

    def move(self, move_val):
        self.current_frame = self.current_frame + move_val
        self.plot_it()


class error_analysis():
    def __init__(self, real_bool, pred_bool, frame_num_array=None):
        if frame_num_array is None:
            frame_num_array = np.asarray([len(real_bool)])
        else:
            # assert 'array' in str(type(frame_num_array)).lower(), 'frame_num_array must be a numpy array or None'
            frame_num_array = np.asarray(frame_num_array).astype(int)
        frame_num_array = list(frame_num_array.astype(int))
        self.frame_nums = frame_num_array
        self.onset_inds_real = utils.search_sequence_numpy(real_bool, np.asarray([0, 1])) + 1
        self.offset_inds_real = utils.search_sequence_numpy(real_bool, np.asarray([1, 0]))
        self.onset_inds_pred = utils.search_sequence_numpy(pred_bool, np.asarray([0, 1])) + 1
        self.offset_inds_pred = utils.search_sequence_numpy(pred_bool, np.asarray([1, 0]))

        # np.concatenate((np.asarray(self.error_lengths_sorted)[self.onset_append_inds], np.asarray(self.error_lengths_sorted)[self.onset_deduct_inds]*-1))
        # np.concatenate((np.asarray(self.error_lengths_sorted)[self.offset_append_inds], np.asarray(self.error_lengths_sorted)[self.offset_deduct_inds]*-1))

        self.correct_onset_inds_sorted = np.intersect1d(self.onset_inds_real, self.onset_inds_pred)
        self.correct_offset_inds_sorted = np.intersect1d(self.offset_inds_real, self.offset_inds_pred)


        # if frame_num_array is None:
        #     frame_num_array = [len(self.pred)]
        self.real = real_bool
        self.pred = pred_bool
        self.type_list = ['ghost', 'ghost', 'append', 'miss', 'miss', 'deduct', 'join', 'split']
        self.type_list_coded = ['ghost', 'miss', 'join', 'split', 'append', 'deduct']
        self.group_inds_neg = []
        self.group_inds_pos = []
        self.error_neg = []
        self.error_pos = []
        self.neg_add_to = []
        self.pos_add_to = []

        self.coded_array = []
        self.e_ghost = []
        self.e_miss = []
        self.e_join = []
        self.e_split = []
        self.e_append = []
        self.e_deduct = []

        self.all_error_type = []
        self.all_errors = []
        self.all_error_nums = []

        self.which_side_test = []


        for i, (i1, i2) in enumerate(self.loop_segments(frame_num_array)):  # separate the trials
            d = self.get_diff_array(self.real[i1:i2], self.pred[i1:i2])  # TP = 2, TN = 0, FP = -1, FN = 1

            R_neg, P_neg, X_neg, group_inds_neg = self.get_error_segments_plus_border(d, -1)

            self.group_inds_neg += group_inds_neg
            for each_x in X_neg:
                self.error_neg.append(self.get_error_type(each_x))
                self.neg_add_to.append(i1)
                self.which_side_test.append([self.one_sided_get_type(each_x), self.one_sided_get_type(np.flip(each_x))])

            R_pos, P_pos, X_pos, group_inds_pos = self.get_error_segments_plus_border(d, 1)
            self.group_inds_pos += group_inds_pos

            for each_x in X_pos:
                self.error_pos.append(self.get_error_type(each_x))
                self.pos_add_to.append(i1)
                self.which_side_test.append([self.one_sided_get_type(each_x), self.one_sided_get_type(np.flip(each_x))])
        #

        self.get_all_errors_final()
        self.get_error_types_separate()

        # get the sorted errors
        tmp1 = []
        for k in self.all_errors:
            tmp1.append(k[0])
        self.all_errors_sorted = []
        self.all_error_type_sorted = []
        self.all_error_nums_sorted = []
        self.which_side_test_sorted = []
        for k in np.argsort(tmp1):
            self.all_errors_sorted.append(self.all_errors[k])
            self.all_error_type_sorted.append(self.all_error_type[k])
            self.all_error_nums_sorted.append(self.all_error_nums[k])
            self.which_side_test_sorted.append(self.which_side_test[k])
        self.error_lengths_sorted = [len(k) for k in self.all_errors_sorted]
        self.append_inds = np.where(np.asarray(self.all_error_type_sorted) == 'append')[0]
        self.deduct_inds = np.where(np.asarray(self.all_error_type_sorted) == 'deduct')[0]
        self.sided_append_or_deduct = np.diff(np.asarray(self.which_side_test_sorted)).flatten()

        self.onset_append_inds = self.append_inds[self.sided_append_or_deduct[self.append_inds] > 0]
        self.offset_append_inds = self.append_inds[self.sided_append_or_deduct[self.append_inds] < 0]
        self.onset_deduct_inds = self.deduct_inds[self.sided_append_or_deduct[self.deduct_inds] > 0]
        self.offset_deduct_inds = self.deduct_inds[self.sided_append_or_deduct[self.deduct_inds] < 0]

        self.correct_onset_inds_sorted = np.intersect1d(self.onset_inds_real, self.onset_inds_pred)
        self.correct_offset_inds_sorted = np.intersect1d(self.offset_inds_real, self.offset_inds_pred)

        self.onset_distance = np.concatenate((np.asarray(self.error_lengths_sorted)[self.onset_append_inds],
                                              np.asarray(self.error_lengths_sorted)[self.onset_deduct_inds] * -1))
        self.offset_distance = np.concatenate((np.asarray(self.error_lengths_sorted)[self.offset_append_inds],
                                               np.asarray(self.error_lengths_sorted)[self.offset_deduct_inds] * -1))

        self.onset_distance = np.concatenate((self.onset_distance, np.zeros_like(self.correct_onset_inds_sorted)))
        self.offset_distance = np.concatenate((self.offset_distance, np.zeros_like(self.correct_offset_inds_sorted)))

    def get_all_errors_final(self):
        for k, i, et in zip(self.group_inds_neg, self.neg_add_to, self.error_neg):
            self.all_errors.append(list(k + i))
            self.all_error_type.append(self.type_list[et])
            self.all_error_nums.append(et)
        for k, i, et in zip(self.group_inds_pos, self.pos_add_to, self.error_pos):
            self.all_errors.append(list(k + i))
            self.all_error_type.append(self.type_list[et])
            self.all_error_nums.append(et)

        # i = np.argsort([k[0] for k in self.all_errors])
        # self.all_errors = [self.all_errors[k] for k in i]
        # self.all_error_type = [self.all_error_type[k] for k in i]
        # self.all_error_nums = [self.all_error_nums[k] for k in i]

    @staticmethod
    def loop_segments(frame_num_array):
        frame_num_array = [0] + list(frame_num_array)
        frame_num_array = np.cumsum(frame_num_array)
        return zip(list(frame_num_array[:-1]), list(frame_num_array[1:]))

    @staticmethod
    def one_sided_get_type(x):
        if np.isnan(x[0]) and np.isnan(x[-1]):
            if x[1] == -1:  # full trial ghost touch
                return -7
            elif x[1] == 1:  # full trial miss touch
                return -4
        if np.isnan(x[0]) or np.isnan(x[-1]):  # edge case (error being parsed is @ index 0 or -1)
            # ## added " or np.isnan(x[-1]):" on september 17th 2022, in case something goes wrong, this makes real-> [0,0,0,1,1,1] ...
            # pred-> [0,0,0,1,1,0] a split instead of a deduct, this is important for TL where length of training data can very ...
            # small and in the middle of a touch
            return -1
        #                                          ghost.   ghost.   append.  miss.   miss.    deduct.
        ind = np.where(np.all(x[:2] == np.asarray([[0, -1], [1, -1], [2, -1], [0, 1], [-1, 1], [2, 1]]), axis=1))[0][0]
        return ind

    @staticmethod
    def type_parser(x):
        minx = np.min(x)
        maxx = np.max(x)
        sort_vals = [minx, maxx]
        if np.any(np.all(sort_vals == np.asarray([[1, 1], [1, 2], [2, 2]]), axis=1)):  # joins
            return 6
        elif maxx == minx and maxx == 5:  # splits
            return 7
        else:
            return maxx  # everything else is already the correct index return the max

    def get_error_type(self, x):
        return self.type_parser([self.one_sided_get_type(x), self.one_sided_get_type(np.flip(x))])

    @staticmethod
    def get_diff_array(real_bool, pred_bool):
        diff_array = np.asarray(real_bool) - np.asarray(pred_bool)
        diff_array = diff_array + (real_bool + pred_bool == 2) * 2  # TP = 2, TN = 0, FP = -1, FN = 1
        return diff_array

    def get_error_segments_plus_border(self, d, error_num_one_or_neg_one):
        group_inds, _ = utils.group_consecutives(np.where(d == error_num_one_or_neg_one)[0])
        R = []
        P = []
        X = []
        for k in group_inds:
            R_ind = k[-1] + 2
            L_tmp = k[0] - 1
            L_ind = np.max([L_tmp, 0])
            #  for this error segment (plus edges) for ...
            r = self.real[L_ind:R_ind]  # real,
            p = self.pred[L_ind:R_ind]  # predicted and
            x = d[L_ind:R_ind]  # the diff array (all match types).

            if L_tmp < 0:  # edge case to the left
                r = np.append(np.nan, r)
                p = np.append(np.nan, p)
                x = np.append(np.nan, x)
            if R_ind > len(d):  # edge case to the right
                r = np.append(r, np.nan)
                p = np.append(p, np.nan)
                x = np.append(x, np.nan)
            R.append(r)
            P.append(p)
            X.append(x)

        return R, P, X, group_inds

    def get_error_types_separate(self):
        """

        Returns
        -------
        self.coded_array where -2 is a correct time point and indexes 0 through 5 match the list of strings
        self.type_list_coded ['ghost', 'miss', 'join', 'split', 'append', 'deduct']
        """
        coded_array = np.zeros_like(self.real) - 2
        all_errors = np.asarray(self.all_errors, dtype=object)
        all_error_type = np.asarray(self.all_error_type)
        for i, k in enumerate(self.type_list_coded):
            tmp2 = all_errors[k == all_error_type]
            all_inds = []
            for kk in tmp2:
                all_inds += kk
            for kk in all_inds:
                coded_array[kk] = i
            exec('self.e_' + k + '=all_inds')
        self.coded_array = coded_array


def name_filt(all_data, filt_string):
    keep_filt = []
    all_data[0].keys()
    for k in range(len(all_data)):
        if filt_string in all_data[k]['full_name']:
            keep_filt.append(k)

    return keep_filt


def name_filt_remove(all_data, filt_string):
    keep_filt = []
    all_data[0].keys()
    for k in range(len(all_data)):
        if filt_string not in all_data[k]['full_name']:
            keep_filt.append(k)

    return keep_filt


def performance_filter(all_data, key_name, greater_than=.8, less_than_or_qual_to=1):
    keep_filt = []
    for i, a in enumerate(all_data):
        log_ind = np.where(key_name == a['logs_names'])[0][0]
        val_list = a['all_logs'][:, log_ind]
        test1 = np.logical_and(val_list > greater_than, val_list <= less_than_or_qual_to).any()
        if test1:
            keep_filt.append(i)
    return keep_filt


def plot_acc(all_data, all_keep_filt, figsize=[15, 10]):
    all_leg = []
    plt.figure(figsize=figsize)
    all_colors = []
    for i in all_keep_filt:
        a = all_data[i]
        key_name = 'acc_val'
        log_ind = np.where(key_name == a['logs_names'])[0][0]

        p = plt.plot(a['all_logs'][:, log_ind])
        all_colors.append(p[0].get_color())
        all_leg.append(a['full_name'] + '_val')

        key_name = 'acc_test'
        log_ind = np.where(key_name == a['logs_names'])[0][0]
        plt.plot(a['all_logs'][:, log_ind], '--', color=all_colors[-1])
        all_leg.append(a['full_name'] + '_test')

    plt.ylim([.8, 1])
    plt.legend(all_leg)
    plt.ylabel('Accuracy', fontsize=22)
    plt.xlabel('Epochs', fontsize=22)


def plot_touch_segment_with_array_blocks(actual_h5_img_file, in_list_of_arrays=[], touch_number=0, border=4, height=20,
                                         img_width=61, color_list=None, cmap_col='inferno'):
    if color_list is None:
        color_list = [0, .5, .2, .3, .75, .85]

    if in_list_of_arrays == []:
        print('no input arrays, returning...')
        return
    color_dict = dict()
    cmap = cm.get_cmap(cmap_col)

    for i, k1 in enumerate(color_list):
        color_dict[i] = np.asarray(cmap(k1)[:-1]) * 255

    in_list_of_arrays = copy.deepcopy(in_list_of_arrays)
    tmp1, tmp2 = utils.group_consecutives(np.where(in_list_of_arrays[0] != 0)[0])

    inds = list(range(tmp1[touch_number][0] - border, tmp1[touch_number][-1] + 1 + border * 2))

    for i, k in enumerate(in_list_of_arrays):
        k = k.astype(float)
        if i == 0:
            tmp1 = np.tile(np.repeat(k[inds], img_width, axis=0), (height, 1))
        else:
            tmp1 = np.vstack((tmp1, np.tile(np.repeat(k[inds], img_width, axis=0), (height, 1))))
    tmp1 = np.stack((tmp1,) * 3, axis=-1)

    for kk in np.unique(tmp1.astype(int)):
        tmp3 = np.where(tmp1 == kk)
        for i1, i2 in zip(tmp3[0], tmp3[1]):
            tmp1[i1, i2, :] = color_dict[kk]

    tmp1 = tmp1.astype(int)
    with h5py.File(actual_h5_img_file, 'r') as h:
        tmp2 = image_tools.img_unstacker(h['images'][inds[0]:inds[-1] + 1], num_frames_wide=len(inds))
        tmp2 = np.vstack((tmp1, tmp2))
        return tmp2

# tmp2 = plot_touch_segment_with_array_blocks(actual_h5_img_file, in_list_of_arrays=[real, pred_m, pred_v],
#                                             touch_number=202, border=4, height=20, img_width=61,
#                                             color_list=[0, .5, .2, .3, .85, .95], cmap_col='inferno')
# plt.figure(figsize=(20, 10))
# plt.imshow(tmp2)



from whacc import utils, image_tools, model_maker
from whacc.pole_tracker import PoleTracking
import os
from whacc.feature_maker import feature_maker, total_rolling_operation_h5_wrapper, convert_h5_to_feature_h5, standard_feature_generation, load_selected_features
import h5py
import numpy as np
import time
import shutil
from pathlib import Path
import pandas as pd
import datetime


# from natsort import natsorted


def batch_process_videos_on_colab(bd, local_temp_dir, video_batch_size=2):
    bd_base_name = os.path.basename(os.path.normpath(bd))
    # load model once in the beginning
    RESNET_MODEL = model_maker.load_final_model()  # NOTE: there will be a warning since this likely isn't optimized for GPU, this is fine
    time_dict = {'num_files': [], 'time_copy_to_local': [], 'time_all': [], 'time_to_track': [], 'time_to_3lag': [],
                 'time_to_features': [], 'time_to_all_features': [], 'time_to_cut_features': []}
    time_df = None
    while True:  # keep going until there are no more files to process

        grab_file_list = True
        while grab_file_list:  # continuously look for files to run
            # get files that tell us which mp4s to process
            list_of_file_dicts = np.asarray(utils.get_files(bd, '*file_list_for_batch_processing.pkl'))
            # sort it by the newest first since we we edit it each time (becoming the newest file)
            # this ensures we finished one set completely first
            inds = np.flip(np.argsort([os.path.getctime(k) for k in list_of_file_dicts]))
            list_of_file_dicts = list_of_file_dicts[inds]
            if len(list_of_file_dicts) == 0:
                print('FINISHED PROCESSING')
                return time_df
                # assert False, "FINISHED PROCESSING no more files to process"
            # load file dictionary
            file_dict = utils.load_obj(list_of_file_dicts[0])
            # get base directory for current videos we are processing
            mp4_bd = os.path.dirname(list_of_file_dicts[0])
            # copy folder structure for the finished mp4s and predictions to go to
            utils.copy_folder_structure(bd, os.path.normpath(bd) + '_FINISHED')
            # check if all the files have already been processes
            if np.all(file_dict['is_processed'] == True):
                x = list_of_file_dicts[
                    0]  # copy the instruction file with list of mp4s to final directory we are finished
                shutil.move(x, x.replace(bd_base_name, bd_base_name + '_FINISHED'))
                x = os.path.dirname(x) + os.sep + 'template_img.png'
                shutil.move(x, x.replace(bd_base_name, bd_base_name + '_FINISHED'))
            else:
                grab_file_list = False  # ready to run data
        start = time.time()
        time_list = [start]
        # overwrite local folder to copy files to
        if os.path.exists(local_temp_dir):
            shutil.rmtree(local_temp_dir)
        Path(local_temp_dir).mkdir(parents=True, exist_ok=True)
        # copy over mp4s and template image to local directory
        x = os.sep + 'template_img.png'
        template_dir = local_temp_dir + x
        shutil.copy(mp4_bd + x, template_dir)
        process_these_videos = np.where(file_dict['is_processed'] == False)[0][:video_batch_size]
        for i in process_these_videos:
            x = os.sep + os.path.basename(file_dict['mp4_names'][i])
            shutil.copy(mp4_bd + x, local_temp_dir + x)
        time_list.append(time.time())  #

        # track the mp4s for the pole images
        PT = PoleTracking(video_directory=local_temp_dir, template_png_full_name=template_dir)
        PT.track_all_and_save()
        time_list.append(time.time())
        # convert the images to '3lag' images
        #  h5_in = '/Users/phil/Desktop/temp_dir/AH0407x160609.h5'
        h5_in = PT.full_h5_name
        h5_3lag = h5_in.replace('.h5', '_3lag.h5')
        image_tools.convert_to_3lag(h5_in, h5_3lag)
        time_list.append(time.time())

        # convert to feature data
        # h5_3lag = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag.h5'
        h5_feature_data = h5_3lag.replace('.h5', '_feature_data.h5')
        in_gen = image_tools.ImageBatchGenerator(500, h5_3lag, label_key='labels')
        convert_h5_to_feature_h5(RESNET_MODEL, in_gen, h5_feature_data)
        time_list.append(time.time())

        # delete 3lag don't it need anymore
        os.remove(h5_3lag)
        # h5_feature_data = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag_feature_data.h5'
        # generate all the modified features (41*2048)+41 = 84,009
        standard_feature_generation(h5_feature_data)
        time_list.append(time.time())
        all_x = load_selected_features(h5_feature_data)
        # delete the big o' file
        file_nums = str(process_these_videos[0] + 1) + '_to_' + str(process_these_videos[-1] + 1) + '_of_' + str(
            len(file_dict['is_processed']))
        h5_final = h5_in.replace('.h5', '_final_to_combine_' + file_nums + '.h5')
        print(h5_final)
        with h5py.File(h5_final, 'w') as h:
            h['final_3095_features'] = all_x
        utils.copy_over_all_non_image_keys(h5_in, h5_final)
        # then if you want you can copy the images too maybe just save as some sort of mp4 IDK
        time_list.append(time.time())
        os.remove(h5_feature_data)
        x = os.path.dirname(list_of_file_dicts[0]) + os.sep
        dst = x.replace(bd_base_name, bd_base_name + '_FINISHED') + os.path.basename(h5_final)
        shutil.copy(h5_final, dst)

        for k in process_these_videos:  # save the dict file so that we know the video has been processed
            file_dict['is_processed'][k] = True
        utils.save_obj(file_dict, list_of_file_dicts[0])

        # move the mp4s to the final dir
        for i in process_these_videos:
            x = mp4_bd + os.sep + os.path.basename(file_dict['mp4_names'][i])
            shutil.move(x, x.replace(bd_base_name, bd_base_name + '_FINISHED'))
        time_list.append(time.time())

        time_array = np.diff(time_list)
        df_name_list = ['copy mp4s to local', 'track the pole', 'convert to 3lag', 'create feature data (CNN)',
                        'engineer all features', 'make final h5 3095', 'copy  h5 and mp4s to final destination',
                        'number of files']

        len_space = ' ' * max(len(k) + 4 for k in df_name_list)
        print('operation                                 total     per file')
        to_print = ''.join([(k2 + len_space)[:len(len_space)] + str(datetime.timedelta(seconds=k)).split(".")[
            0] + '   ' + str(datetime.timedelta(seconds=k3)).split(".")[0] + '\n' for k, k2, k3 in
                            zip(time_array, df_name_list, time_array / len(process_these_videos))])
        print(to_print)
        print('TOTAL                                     total     per file')
        print(len_space + str(datetime.timedelta(seconds=np.sum(time_array))).split(".")[0] + '   ' +
              str(datetime.timedelta(seconds=np.sum(time_array / len(process_these_videos)))).split(".")[0])

        time_array = np.concatenate((time_array, [len(process_these_videos)]))
        if time_df is None:
            time_df = pd.DataFrame(time_array[None, :], columns=df_name_list)
        else:
            tmp_df = pd.DataFrame(time_array[None, :], columns=df_name_list)
            time_df = time_df.append(tmp_df, ignore_index=True)


# time_df = pd.DataFrame(time_array[None, :], columns=df_name_list)
#
# time_df2 = pd.DataFrame(time_array[None, :]*2, columns=df_name_list)
#
# df = time_df.append(time_df2, ignore_index = True)
#
#
# len_space = ' ' * max(len(k)+4 for k in df_name_list)
# print('operation                                 total     per file')
# to_print = ''.join([(k2+ len_space)[:len(len_space)] + str(datetime.timedelta(seconds=k)).split(".")[0] + '   ' + str(datetime.timedelta(seconds=k3)).split(".")[0] + '\n' for k, k2, k3 in
#  zip(time_array, df_name_list, time_array/len(process_these_videos))])
# print(to_print)
# print('TOTAL                                     total     per file')
# print(len_space+ str(datetime.timedelta(seconds=np.sum(time_array))).split(".")[0]+ '   ' + str(datetime.timedelta(seconds=np.sum(time_array/len(process_these_videos)))).split(".")[0])
#
# print(df.iloc[-1])
#
#
# time_array = np.diff([1650320439.4598331,
#  1650320439.6510222,
#  1650320482.45639,
#  1650320519.597101,
#  1650320602.7472188,
#  1650320666.029782,
#  1650320670.40395,
#  1650320671.3225741])
# time_array = np.concatenate((time_array, [2]))


"""
prior to this you will need a template image in each of your MP4 directories 
and you must have run the following on the complete i.e. -- no missing mp4s due to them still uploading for example, 
so this is best to do on local then upload. bu if all are already uploaded can do this just as easily on the cloud. 

base_directory = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here/'
folders_with_MP4s = whacc.utils.recursive_dir_finder(base_directory, '*.mp4')
for video_directory in folders_with_MP4s:
    utils.make_mp4_list_dict(video_directory) # this creates all the a file for batch processing to keep track of every 
    # folder with MP4s to process  
"""

utils.make_mp4_list_dict('/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here/Jon/AH0407/160613', overwrite=True) #OUSD*AUOINJFNIUAHSFIUHIUH
utils.make_mp4_list_dict('/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here/Phil/AH0698/170601', overwrite=True) #OUSD*AUOINJFNIUAHSFIUHIUH



video_batch_size = 5
# local colab folder, or other local folder.
local_temp_dir = '/Users/phil/Desktop/temp_dir'
# base_processing_folder
bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here/'

time_df = utils.batch_process_videos_on_colab(bd, local_temp_dir, video_batch_size=video_batch_size)




b2_back_one = os.path.dirname(os.path.normpath(bd))  # will look one directory back
utils.auto_combine_final_h5s(b2_back_one)  # if length is one and 0 through end then just rename it







# from whacc import PoleTracking
video_directory = '/Users/phil/Dropbox/HIRES_LAB/curation_for_auto_curator/Data/Samson/Session322/'
PT = PoleTracking(video_directory=video_directory)  # create the 'PoleTracking' class
PT.cut_out_pole_template(crop_size=[71] * 2, frame_num=2000, file_ind=2)  # user cut out template image
PT.save_template_img(cust_save_dir=PT.video_directory)  # save the template image

from whacc import utils

tmp1 = '/Users/phil/Dropbox/HIRES_LAB/GitHub/whacc/whacc/whacc_data/feature_data/feature_data_dict.pkl'
d = utils.load_obj(tmp1)

#
# from whacc import PoleTracking, utils, image_tools, model_maker, feature_maker
# import os
# from whacc.feature_maker import feature_maker, total_rolling_operation_h5_wrapper, convert_h5_to_feature_h5, standard_feature_generation, load_selected_features
# import h5py
# import numpy as np
# import time
# import shutil
# from pathlib import Path
# from natsort import natsorted
#
# ################
# ################ USER SETTINGS BELOW
# ################
# video_batch_size = 5
# # local colab folder, or other local folder.
# local_temp_dir = '/Users/phil/Desktop/temp_dir'
# # base_processing_folder
# bd = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test/add_files_here/'
# bd_base_name = os.path.basename(os.path.normpath(bd))
# ################
# ################ USER SETTINGS ABOVE
# ################
# # load model once in the beginning
# RESNET_MODEL = model_maker.load_final_model()  # NOTE: there will be a warning since this likely isn't optimized for GPU, this is fine
# fd = utils.load_feature_data()  # load feature data info
# while True:  # once all files are
#     time_list = []
#     start = time.time()
#     grab_file_list = True
#     while grab_file_list:  # continuously look for files to run
#         # get files that tell us which mp4s to process
#         list_of_file_dicts = np.asarray(utils.get_files(bd, '*file_list_for_batch_processing.pkl'))
#         # sort it by the newest first since we we edit it each time (becoming the newest file)
#         # this ensures we finished one set completely first
#         inds = np.flip(np.argsort([os.path.getctime(k) for k in list_of_file_dicts]))
#         list_of_file_dicts = list_of_file_dicts[inds]
#         if len(list_of_file_dicts) == 0:
#             print('FINISHED PROCESSING')
#             assert False, "FINISHED PROCESSING no more files to process"
#         # load file dictionary
#         file_dict = utils.load_obj(list_of_file_dicts[0])
#         # get base directory for current videos we are processing
#         mp4_bd = os.path.dirname(list_of_file_dicts[0])
#         # copy folder structure for the finished mp4s and predictions to go to
#         utils.copy_folder_structure(bd, os.path.normpath(bd) + '_FINISHED')
#         # check if all the files have already been processes
#         if np.all(file_dict['is_processed'] == True):
#             x = list_of_file_dicts[0]  # copy the instruction file with list of mp4s to final directory we are finished
#             shutil.move(x, x.replace(bd_base_name, bd_base_name + '_FINISHED'))
#             x = os.path.dirname(x) + os.sep + 'template_img.png'
#             shutil.move(x, x.replace(bd_base_name, bd_base_name + '_FINISHED'))
#         else:
#             grab_file_list = False  # ready to run data
#
#     # overwrite local folder to copy files to
#     if os.path.exists(local_temp_dir):
#         shutil.rmtree(local_temp_dir)
#     Path(local_temp_dir).mkdir(parents=True, exist_ok=True)
#     # copy over mp4s and template image to local directory
#     x = os.sep + 'template_img.png'
#     template_dir = local_temp_dir + x
#     shutil.copy(mp4_bd + x, template_dir)
#     process_these_videos = np.where(file_dict['is_processed'] == False)[0][:video_batch_size]
#     for i in process_these_videos:
#         x = os.sep + os.path.basename(file_dict['mp4_names'][i])
#         shutil.copy(mp4_bd + x, local_temp_dir + x)
#
#     # track the mp4s for the pole images
#     PT = PoleTracking(video_directory=local_temp_dir, template_png_full_name=template_dir)
#     PT.track_all_and_save()
#
#     # convert the images to '3lag' images
#     #  h5_in = '/Users/phil/Desktop/temp_dir/AH0407x160609.h5'
#     h5_in = PT.full_h5_name
#     h5_3lag = h5_in.replace('.h5', '_3lag.h5')
#     image_tools.convert_to_3lag(h5_in, h5_3lag)
#
#     # convert to feature data
#     # h5_3lag = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag.h5'
#     h5_feature_data = h5_3lag.replace('.h5', '_feature_data.h5')
#     in_gen = image_tools.ImageBatchGenerator(500, h5_3lag, label_key='labels')
#     convert_h5_to_feature_h5(RESNET_MODEL, in_gen, h5_feature_data)
#
#     # delete 3lag don't it need anymore
#     os.remove(h5_3lag)
#     # h5_feature_data = '/Users/phil/Desktop/temp_dir/AH0407x160609_3lag_feature_data.h5'
#     # generate all the modified features (41*2048)+41 = 84,009
#     standard_feature_generation(h5_feature_data)
#     all_x = load_selected_features(h5_feature_data)
#     # delete the big o' file
#     file_nums = str(process_these_videos[0] + 1) + '_to_' + str(process_these_videos[-1] + 1) + '_of_' + str(
#         len(file_dict['is_processed']))
#     h5_final = h5_in.replace('.h5', '_final_to_combine_' + file_nums + '.h5')
#     print(h5_final)
#     with h5py.File(h5_final, 'w') as h:
#         h['final_3095_features'] = all_x
#     utils.copy_over_all_non_image_keys(h5_in, h5_final)
#     # then if you want you can copy the images too maybe just save as some sort of mp4 IDK
#     os.remove(h5_feature_data)
#     x = os.path.dirname(list_of_file_dicts[0]) + os.sep
#     dst = x.replace(bd_base_name, bd_base_name + '_FINISHED') + os.path.basename(h5_final)
#     shutil.copy(h5_final, dst)
#
#     for k in process_these_videos:  # save the dict file so that we know the video has been processed
#         file_dict['is_processed'][k] = True
#     utils.save_obj(file_dict, list_of_file_dicts[0])
#
#     # move the mp4s to the final dir
#     for i in process_these_videos:
#         x = mp4_bd + os.sep + os.path.basename(file_dict['mp4_names'][i])
#         shutil.move(x, x.replace(bd_base_name, bd_base_name + '_FINISHED'))
#     end = time.time()
#     time_list.append(end - start)
#
#
#



import copy
import matplotlib.pyplot as plt

import numpy as np
from scipy.signal import medfilt, medfilt2d

from whacc import image_tools
from whacc import utils
import pandas as pd
import seaborn as sns
from matplotlib.lines import Line2D

def foo_rename(instr):
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]


def medfilt_confidence_scores(pred_bool_in, kernel_size_in):
    if len(pred_bool_in.shape) == 1 or pred_bool_in.shape[1] == 1:
        pred_bool_out = medfilt(copy.deepcopy(pred_bool_in).flatten(), kernel_size=kernel_size_in)
    else:
        pred_bool_out = medfilt2d(copy.deepcopy(pred_bool_in), kernel_size=[kernel_size_in, 1])
    return pred_bool_out


def confidence_score_to_class(pred_bool_in, thresh_in):
    if len(pred_bool_in.shape) == 1 or pred_bool_in.shape[1] == 1:
        pred_bool_out = ((pred_bool_in > thresh_in) * 1).flatten()
    else:
        pred_bool_out = np.argmax(pred_bool_in, axis=1)
    #     NOTE: threshold is not used for the multi class models
    return pred_bool_out


def foo_arg_max_and_smooth(pred_bool_in, kernel_size_in, thresh_in, key_name_in, L_key_=None, L_type_split_ind=None):
    pred_bool_out = medfilt_confidence_scores(pred_bool_in, kernel_size_in)
    pred_bool_out = confidence_score_to_class(pred_bool_out, thresh_in)
    if L_key_ is None:
        L_key_ = '_'.join(key_name_in.split('__')[L_type_split_ind].split(' '))

    pred_bool_out = utils.convert_labels_back_to_binary(pred_bool_out, L_key_)
    return pred_bool_out


"""##################################################################################################################"""
"""##################################################################################################################"""
key_name = 'model_5____3lag__regular_labels__MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC_LIGHT_GBM'
h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED/AH0407_160613_JC1003_AAAC_ALT_LABELS.h5'

kernel_size = 7
threshold = 0.5
L_key_ = '[0, 1]- (no touch, touch)'

"""##################################################################################################################"""
"""##################################################################################################################"""
pred_bool_temp = image_tools.get_h5_key_and_concatenate(h5_file, key_name)
pred_bool_temp = pred_bool_temp.astype(float)
pred_bool_smoothed = foo_arg_max_and_smooth(pred_bool_temp, kernel_size, threshold, key_name, L_key_=L_key_)

real_bool = image_tools.get_h5_key_and_concatenate(h5_file, '[0, 1]- (no touch, touch)')
trial_nums_and_frame_nums = image_tools.get_h5_key_and_concatenate(h5_file, 'trial_nums_and_frame_nums')
frame_nums = trial_nums_and_frame_nums[1, :].astype(int)

in_range = image_tools.get_h5_key_and_concatenate(h5_file, 'in_range')
"""##################################################################################################################"""
"""##################################################################################################################"""

all_h5s = utils.get_h5s(foo_rename('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/'),
                        print_h5_list=False)
h_cont, h_names = utils._get_human_contacts_(all_h5s)
"""##################################################################################################################"""
"""##################################################################################################################"""
human = h_cont[0][:, in_range.astype(bool)]
whacc = pred_bool_smoothed[in_range.astype(bool)]

human_mean = np.mean(human, axis=0)


def foo_agree_0_1(in1, in2):
    full0agree = np.mean(1 * (in1 == 0) == 1 * (in2 == 0))
    full1agree = np.mean(1 * (in1 == 1) == 1 * (in2 == 1))
    return full0agree, full1agree


# np.mean(full0agree)*100, np.mean(full1agree)*100
d = {'h0': [], 'h1': [],'w0': [], 'w1': []}
for i, a in enumerate(human):
    inds = np.ones(human.shape[0]).astype(bool)
    # inds[i] = False
    tmp_mean = np.mean(human[inds], axis=0)
    tmp1, tmp2 = foo_agree_0_1(a, tmp_mean)
    d['h0'].append(tmp1)
    d['h1'].append(tmp2)
    tmp1, tmp2 = foo_agree_0_1(whacc, tmp_mean)
    d['w0'].append(tmp1)
    d['w1'].append(tmp2)

df = pd.DataFrame(d)

ax = sns.pointplot(x="h0", y="w0", data=df)




# np.mean(full0agree)*100, np.mean(full1agree)*100
d = {'percent correct':[], 'ind':[], 'label':[], 'class':[]}
for i, a in enumerate(human):
    inds = np.ones(human.shape[0]).astype(bool)
    inds[i] = False
    tmp_mean = np.mean(human[inds], axis=0)
    d['percent correct']+=list(foo_agree_0_1(a, tmp_mean))
    d['percent correct']+=list(foo_agree_0_1(whacc, tmp_mean))
    d['ind']+=[i]*4
    d['label']+=['human', 'human', 'WhACC', 'WhACC']
    d['class']+=['no touch', 'touch']*2

df = pd.DataFrame(d)
plt.figure(figsize=(7, 6))
ax = sns.pointplot(x="label", y="percent correct", hue="class", data=df, dodge=True)

plt.figure(figsize=(7, 6))
ax = sns.pointplot(x="label", y="percent correct", data=df, dodge=True)

plt.figure(figsize=(7, 6))
for k in range(3):
    ax = sns.pointplot(x="label", y="percent correct", data=df.loc[df['ind'] == k], dodge=True, ci=None)




def foo_agree_all(in1, in2):
    return np.mean(np.append([1 * (in1 == 0) == 1 * (in2 == 0)], [1 * (in1 == 1) == 1 * (in2 == 1)]))

d = {'percent correct':[], 'ind':[], 'label':[], 'class':[]}
for i, a in enumerate(human):
    inds = np.ones(human.shape[0]).astype(bool)
    inds[i] = False
    tmp_mean = np.mean(human[inds], axis=0)
    d['percent correct'].append(foo_agree_all(a, tmp_mean))
    d['percent correct'].append(foo_agree_all(whacc, tmp_mean))
    d['ind']+=[i]*2
    d['label']+=['human', 'WhACC']
    d['class']+=['all']*2


df = pd.DataFrame(d)
plt.figure(figsize=(7, 6))
ax = sns.pointplot(x="label", y="percent correct", data=df, dodge=True, ci = False, color='k', legend='sadf')

for k in range(3):
    ax = sns.pointplot(x="label", y="percent correct", data=df.loc[df['ind'] == k], dodge=True, ci=None, color='b')

custom_lines = [Line2D([0], [0], color='k', lw=4),
                Line2D([0], [0], color='b', lw=4)]
plt.legend(custom_lines, ['mean', 'individual'])



from whacc import PoleTracking, utils, image_tools, model_maker, feature_maker
import whacc
import os
import matplotlib.pyplot as plt
from whacc.feature_maker import feature_maker, total_rolling_operation_h5_wrapper, convert_h5_to_feature_h5
from tqdm.auto import tqdm
import numpy as np
import time
import glob
from natsort import natsorted
from whacc.touch_curation_GUI import touch_gui

"""
NOTES TO DO 
ensure each has an overwrite option just delete file if exists dont want to 
retrack things on accident that would waste time

"""
"""######## MAKE H5s FROM MP4s ########"""

time_dict = dict()
start = time.time()
# this will use the pole tracker on all subdirectories, if you want to do it on one directory
# just set ... video_directory = ['your/full.directory/here/']
mp4_path = '/Users/phil/Desktop/pipeline_test/'
mp4_path = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_batch_processing_test'
search_term = '*.mp4'
folders_with_MP4s = whacc.utils.recursive_dir_finder(mp4_path, search_term)
print(folders_with_MP4s, sep='\n')

for video_directory in folders_with_MP4s:
    utils.make_mp4_list_dict(video_directory)

PT = dict()
for i, video_directory in enumerate(folders_with_MP4s):
    template_img_full_name = glob.glob(video_directory + os.path.sep + '*template_img.png')
    if len(template_img_full_name) == 1:  # if the template already exists
        PT[i] = PoleTracking(video_directory=video_directory, template_png_full_name=template_img_full_name[0])
    else:
        PT[i] = PoleTracking(video_directory=video_directory)  # create the 'PoleTracking' class
        PT[i].cut_out_pole_template(crop_size=[61, 61], frame_num=2000, file_ind=2)  # user cut out template image
        PT[i].save_template_img(cust_save_dir=PT[i].video_directory)  # save the template image

for i in PT: #plot image and display trial numbers extract to ensure they match
    PT_tmp = PT[i]
    PT_tmp.get_trial_and_file_names(print_them=True, num_to_print=5)
    PT_tmp.save_template_img(cust_save_dir=PT_tmp.video_directory)
    plt.figure()
    plt.title(PT_tmp.video_directory)
    plt.imshow(PT_tmp.template_image)

for i in PT:
    output_h5 = PT[i].track_all_and_save()

end = time.time()
time_dict['tracking time'] = end - start
start = time.time()
"""######## MAKE 3lag HF FILES - do this for each h5 generated above, only one is illustrated here ########"""
f = '/Users/phil/Desktop/pipeline_test/AH0407x160609.h5'
h5_3lag = f.replace('.h5', '_3lag.h5')
image_tools.convert_to_3lag(f, h5_3lag)
# utils.print_h5_keys(h5_3lag)
# utils.print_h5_keys(f)
end = time.time()
time_dict['3lag time'] = end - start
start = time.time()
# touch_gui(h5_3lag, 'labels', 'labels') # if you want to view your h5 data

h5_feature_data = h5_3lag.replace('.h5', '_feature_data.h5')
mod = model_maker.load_final_model()
in_gen = image_tools.ImageBatchGenerator(500, h5_3lag, label_key='labels')
convert_h5_to_feature_h5(mod, in_gen, h5_feature_data)

utils.print_h5_keys(h5_feature_data)
end = time.time()
time_dict['convert to features time'] = end - start
start = time.time()





##########################################################################################
##################### NOW ENGINEER LAG SMOOTH SD FEATURES ETC#############################
##########################################################################################
##########################################################################################


FM = feature_maker(h5_feature_data, operational_key='FD__original', delete_if_exists=True)

for periods in tqdm([-5, -4, -3, -2, -1, 1, 2, 3, 4, 5]):
    data, key_name = FM.shift(periods, save_it=True)

for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
    data, key_name = FM.rolling(smooth_it_by, 'mean', save_it=True)

for periods in tqdm([-50, -20, -10, -5, -4, -3, -2, -1, 1, 2, 3, 4, 5, 10, 20, 50]):
    data, key_name = FM.operate('diff', kwargs={'periods': periods}, save_it=True)

for smooth_it_by in tqdm([3, 7, 11, 15, 21, 41, 61]):
    data, key_name = FM.rolling(smooth_it_by, 'std', save_it=True)

win = 1
# key_to_operate_on = 'FD__original'
op = np.std
mod_key_name = 'FD_TOTAL_std_' + str(win) + '_of_'
all_keys = utils.lister_it(utils.print_h5_keys(FM.h5_in, 1, 0), 'FD__', 'FD_TOTAL')
for key_to_operate_on in tqdm(all_keys):
    data_out = total_rolling_operation_h5_wrapper(FM, win, op, key_to_operate_on, mod_key_name = mod_key_name, save_it = True)

end = time.time()
time_dict['engineer features time'] = end - start

seconds_per_video = np.sum([291.3832380771637, 242.00185012817383, 545.0492000579834, 606.5919070243835])/10
import datetime
print(str(datetime.timedelta(seconds=seconds_per_video)))

print(str(datetime.timedelta(seconds=5040001.2)))




import matplotlib.pyplot as plt
from tqdm.auto import tqdm
from imgaug import augmenters as iaa  # optional program to further augment data

from whacc import utils
import numpy as np
from whacc import image_tools
from natsort import natsorted, ns
import pickle
import pandas as pd
import os
import copy
import seaborn as sns
from keras.preprocessing.image import ImageDataGenerator
import h5py

from whacc import utils
import h5py
import matplotlib.pyplot as plt
import numpy as np
from pathlib import Path

"""
################################################################################################
################################################################################################
################################################################################################
################################################################################################
"""
#  match the sessions


#
"""
################################################################################################
################################################################################################
################################################################################################
################################################################################################
"""

bd_fileLists = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1jFuXGPmP8QNNZxuCzCA8qVItvPJvg8hU/contactsMatter/fileLists/'
bd_binarizedContacts_All = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1jFuXGPmP8QNNZxuCzCA8qVItvPJvg8hU/contactsMatter/binarizedContacts_All/'
save_dir = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1jFuXGPmP8QNNZxuCzCA8qVItvPJvg8hU/contactsMatter/dictionary_saves/'
utils.make_path(save_dir)

final_dict = dict()
bd_fileLists = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1jFuXGPmP8QNNZxuCzCA8qVItvPJvg8hU/contactsMatter/fileLists'
f_list = utils.sort(utils.get_files(bd_fileLists, '*.mat'))
for f in tqdm(f_list):
    d = dict()
    tmp1 = utils.loadmat(f)
    fileList = tmp1['fileList']
    d['file_list'] = fileList
    add_name = os.path.basename(f).split('_fileList.mat')[0]
    contact_dir = bd_binarizedContacts_All + add_name
    all_contacts = []
    frame_nums = []
    contact_names = utils.sort(utils.get_files(contact_dir, '*.mat'))
    d['contact_names'] = contact_names

    for k in contact_names:
        tmp2 = utils.loadmat(k)['binarizedContacts']
        frame_nums.append(len(tmp2))
        all_contacts.append(tmp2)
    d['contacts'] = all_contacts
    d['frame_nums'] = frame_nums

    save_name = save_dir + add_name + '.pkl'
    utils.save_obj(d, save_name)
    final_dict[add_name] = d

save_name = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1jFuXGPmP8QNNZxuCzCA8qVItvPJvg8hU/contactsMatter/final_dict.pkl'
utils.save_obj(final_dict, save_name)

final_dict = utils.load_obj(save_name)
"""
final_dict is a dict of dicts, each inside dict is a session each ones of those has the following 

contact_names type->   list           ...ntacts_All/Session14/Session14_100.mat']
contacts      type->   list           ...y([0, 0, 0, ..., 0, 0, 0], dtype=uint8)]
frame_nums    type->   list           ...000, 3000, 3000, 3000, 3000, 3000, 3000]
file_list     type->   numpy.ndarray  ...\Session14\\AH1179X01062021x14-100.mp4']

utils.info(final_dict['Session1'])
"""


def smooth(y, box_pts):  # $%
    box = np.ones(box_pts) / box_pts
    y_smooth = np.convolve(y, box, mode='same')
    return y_smooth


def h5_to_dict(h5_in):
    d = dict()
    with h5py.File(h5_in, 'r') as h:
        for k in h.keys():
            d[k] = h[k][:]
    return d


T_V_Test_dict = [{'file_list': [], 'contact_names': [], 'contacts': [], 'frame_nums': [], 'session': [],
                  'session_index': [],
                  '80_border_inds': [], '20_border_inds': [], '10_border_inds': [], '3_border_inds': []}] * 3

for k in final_dict.keys():  # k are key strings names 'session1' etc
    L = len(final_dict[k]['frame_nums'])
    np.random.seed(0)
    T_V_Test_inds = utils.split_list_inds(range(L), [6, 2, 2])
    final_dict[k]['T_V_Test_inds'] = T_V_Test_inds
    for ind1, kk in enumerate(T_V_Test_inds):  # ind1 is 0 1 or 2 or t v and test, kk are the file inds
        for i in kk:
            T_V_Test_dict[ind1]['session_index'].append(i)
            T_V_Test_dict[ind1]['session'].append(k)
            T_V_Test_dict[ind1]['file_list'].append(final_dict[k]['file_list'][i])
            T_V_Test_dict[ind1]['contact_names'].append(final_dict[k]['contact_names'][i])
            T_V_Test_dict[ind1]['frame_nums'].append(final_dict[k]['frame_nums'][i])
            T_V_Test_dict[ind1]['contacts'].append(final_dict[k]['contacts'][i])
            for border_tmp in [80, 20, 10, 3]:
                border = border_tmp * 2 + 1
                tmp2 = smooth(T_V_Test_dict[ind1]['contacts'][-1], border)
                good_inds = tmp2 > 0
                T_V_Test_dict[ind1][str(border_tmp) + '_border_inds'].append(good_inds)
                """
                so it looks like the 80 border used the list of contacts from its own subset selection (i.e. training validation test)
                so I will need to 
                so it randomizes the files into trianing validation and test 
                then makes contact, files list etc for those mixed sets 
                then for those contacts (that are mixed and indexed) it takes teh 80 border for those meaning that the index 
                of the 80 border 
                """

for k in T_V_Test_dict:
    print(len(k['contacts']))
    # print(np.sum(np.concatenate(k['80_border_inds'])))

save_name = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1jFuXGPmP8QNNZxuCzCA8qVItvPJvg8hU/contactsMatter/final_T_V_Test_dict.pkl'
utils.save_obj(T_V_Test_dict, save_name)

T_V_Test_dict = utils.load_obj(save_name)

utils.info(T_V_Test_dict[0])

utils.info(T_V_Test_dict[0]['session'])

final_dict['Session1']['file_list'][:4]

h5_bd_session = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/'
# fn = []
for set_dict in T_V_Test_dict:  # list of dicts each with all info needed to make sets
    # from set_dict I need the session and the video number that matches
    for sess in np.unique(set_dict['session']):  # ook through all the session
        h5_list = utils.sort(utils.get_h5s(h5_bd_session + sess))  # get list of h5s per each session
        each_h5_inds = [int(os.path.basename(k).split('-')[-1][:-4]) for k in set_dict['file_list'][:]]
        sess_inds = np.asarray(set_dict['session'][:]) == sess
        for h5 in h5_list:  # for now just session 1 TEMP
            with h5py.File(h5, 'r') as h:
                full_file_names = [n.decode("ascii", "ignore") for n in h['full_file_names'][:]]
                file_name_nums = h['file_name_nums'][:]
                np.unique(file_name_nums)

                final_features_2105 = h['final_features_2105'][:]

                print(h.keys())
                fn = h['frame_nums'][:]
                asdf

h = h5_to_dict(h5)
utils.info(h)
full_file_names = [n.decode("ascii", "ignore") for n in h['full_file_names'][:]]

utils.info(set_dict)

set_dict['file_list'][-1]
set_dict['contact_names'][-1]

all_features = []
for h5 in tqdm(h5_list):
    with h5py.File(h5, 'r') as h:
        all_features.append(h['final_features_2105'][:])


def combine_final_h5s(h5_file_list_to_combine, delete_extra_files=False):
    fn = h5_file_list_to_combine[0].split('final')[0] + 'final_combined.h5'
    h5c = image_tools.h5_iterative_creator(fn,
                                           overwrite_if_file_exists=True,
                                           max_img_height=1,
                                           max_img_width=2105,
                                           close_and_open_on_each_iteration=True,
                                           color_channel=False,
                                           add_to_existing_H5=False,
                                           ignore_image_range_warning=False,
                                           dtype_img=h5py.h5t.IEEE_F32LE,
                                           dtype_labels=h5py.h5t.IEEE_F32LE,
                                           image_key_name='final_features_2105')

    for k in tqdm(h5_file_list_to_combine):
        with h5py.File(k, 'r') as h:
            # print(h.keys())
            final_features_2105 = h['final_features_2105'][:]
            h5c.add_to_h5(final_features_2105, np.ones(final_features_2105.shape[0]) * -1)
    with h5py.File(fn, 'r+') as h:
        del h['labels']

    keys = ['file_name_nums', 'frame_nums', 'full_file_names', 'in_range', 'labels',
            'locations_x_y', 'max_val_stack']

    trial_nums_and_frame_nums = []
    for k in h5_file_list_to_combine:
        print(k)
        trial_nums_and_frame_nums.append(image_tools.get_h5_key_and_concatenate(str(k), 'trial_nums_and_frame_nums'))
    trial_nums_and_frame_nums = np.hstack(trial_nums_and_frame_nums)

    with h5py.File(fn, 'r+') as h:
        for k in keys:
            print(k)
            out = image_tools.get_h5_key_and_concatenate(h5_file_list_to_combine, k)
            h[k] = out

    utils.overwrite_h5_key(fn, 'template_img',
                           image_tools.get_h5_key_and_concatenate(h5_file_list_to_combine[0], 'template_img'))
    utils.overwrite_h5_key(fn, 'multiplier',
                           image_tools.get_h5_key_and_concatenate(h5_file_list_to_combine[0], 'multiplier'))
    utils.overwrite_h5_key(fn, 'trial_nums_and_frame_nums',
                           image_tools.get_h5_key_and_concatenate(h5_file_list_to_combine[0],
                                                                  'trial_nums_and_frame_nums'))
    if delete_extra_files:
        for k in h5_file_list_to_combine:
            os.remove(k)


h5_bd_session = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/'
# h5_bd_session = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/untitled folder/'
h5_file_list_to_combine = [str(k) for k in
                           utils.sort(utils.lister_it(utils.get_h5s(h5_bd_session), 'final_to_combine'))]
combine_final_h5s(h5_file_list_to_combine, delete_extra_files=False)

tmp1 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/AH1179X23052021x1_final_combined.h5'
with h5py.File(tmp1, 'r') as h:
    for k in h.keys():
        print(k)
        try:
            print(h[k][:].shape)
        except:
            pass
        print('___')
print('___')
print('___')
print('___')
tmp1 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/AH1179X23052021x1_final_to_combine_6_to_10_of_299.h5'
with h5py.File(tmp1, 'r') as h:
    for k in h.keys():
        print(k)
        try:
            print(h[k][:].shape)
        except:
            pass
        print('___')

h5_bd_session = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/'
utils.auto_combine_final_h5s(h5_bd_session, False)

utils.print_h5_keys(
    '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/untitled folder/AH1179X23052021x1_final_combined.h5')
utils.print_h5_keys(
    '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/untitled folder/AH1179X23052021x1_final_to_combine_1_to_5_of_299.h5')

k = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/untitled folder/AH1179X23052021x1_final_to_combine_1_to_5_of_299.h5'
k = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/untitled folder/AH1179X23052021x1_final_to_combine_1_to_5_of_299.h5'
utils.print_h5_keys(k)
trial_nums_and_frame_nums = []
trial_nums_and_frame_nums.append(image_tools.get_h5_key_and_concatenate(k, 'trial_nums_and_frame_nums'))

trial_nums_and_frame_nums = []
for k in h5_file_list_to_combine:
    print(k)
    trial_nums_and_frame_nums.append(image_tools.get_h5_key_and_concatenate(k, 'trial_nums_and_frame_nums'))

utils.auto_combine_final_h5s(h5_bd_session, False)

h5 = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/Session1_FINISHED/Session1/AH1179X23052021x1_final_combined.h5'
utils.print_h5_keys(h5)
# match the files with the contacts
utils.info(T_V_Test_dict[0])

file_name_nums_from_features = np.asarray(
    [int(os.path.basename(k.decode("ascii", "ignore")).split('-')[-1][:-4]) for k in
     image_tools.get_h5_key_and_concatenate(h5, 'full_file_names')])

save_name = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1jFuXGPmP8QNNZxuCzCA8qVItvPJvg8hU/contactsMatter/final_T_V_Test_dict.pkl'
T_V_Test_dict = utils.load_obj(save_name)

import pdb


def foo_save_data(save_bd, features, labels):
    utils.make_path(os.path.dirname(save_bd))
    features = np.vstack(features)
    labels = np.concatenate(labels)
    np.save(save_bd + '_features', features)
    np.save(save_bd + '_labels', labels)


border_key = '80_border_inds'
# get same session, match all the contacts

bd = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/'
save_bd = bd + '/samson_model_data/'

cnt = 0
save_cnt = 0
save_every_x = 50
dataset = T_V_Test_dict[0]
for (dataset, dataset_name) in zip(T_V_Test_dict, ['train', 'val', 'test']):
    for i in range(len(dataset['file_list'])):
        f = dataset['file_list'][i]
        f = utils.norm_path(f)
        fn_cont = dataset['frame_nums'][i]
        contacts = dataset['contacts'][i]
        border_inds = dataset[border_key][i]
        sess = os.path.basename(os.path.dirname(f))

        # h5 from matching session
        h5 = bd + sess + '_FINISHED/' + sess
        h5 = utils.get_files(h5, '*final_combined.h5')
        assert len(h5) == 1, 'asdfasdf'
        h5 = h5[0]

        frame_nums_from_features = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
        # below matches the contact SESSION already
        file_name_nums_from_features = np.asarray(
            [int(os.path.basename(k.decode("ascii", "ignore")).split('-')[-1][:-4]) for k in
             image_tools.get_h5_key_and_concatenate(h5, 'full_file_names')])
        file_name_num_from_contacts = int(os.path.basename(f.split('-')[-1][:-4]))
        ind = np.where(file_name_num_from_contacts == file_name_nums_from_features)[0]  # ind to the feature data

        if len(ind) == 0:
            print('no match\n')
            # pass # no match
        elif len(ind) > 1:
            assert False, 'more than one match this is impossible'
        else:

            if cnt // save_every_x == cnt / save_every_x:
                if save_cnt > 0:
                    foo_save_data(save_bd + '/' + dataset_name + '/' + str(save_cnt).zfill(3), final_features_2105_TEMP,
                                  contact_TEMP)
                    pdb.set_trace()
                save_cnt += 1
                final_features_2105_TEMP = []
                contact_TEMP = []
                cnt = 0
            cnt += 1
            ind = ind[0]
            loop_segs = np.asarray(utils.loop_segments(frame_nums_from_features, True))
            with h5py.File(h5, 'r') as h:
                final_features_2105 = h['final_features_2105'][loop_segs[0, ind]:loop_segs[1, ind]]
            # final_features_2105 = image_tools.get_h5_key_and_concatenate(h5, 'final_features_2105')
            fn_f = frame_nums_from_features[ind]
            assert fn_f == fn_cont, 'frame number miss match WTF'
            contact_TEMP.append(contacts[border_inds])
            final_features_2105_TEMP.append(final_features_2105[border_inds, :])
            print(len(contacts[border_inds]))
    if cnt != save_every_x:  # save remainder files
        foo_save_data(save_bd + '/' + dataset_name + '/' + str(save_cnt).zfill(3), final_features_2105_TEMP,
                      contact_TEMP)

np.asarray(tmp1)
loop_segs = np.asarray(utils.loop_segments(frame_nums_from_features, True))

"""
chekc if file synced for samsons data

24 core convert all of the h5s into a single H5 file. 
"""

cnt1 = 0
cnt2 = 0
bd = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/'
from whacc.utils import get_files, load_obj, sort

finished_sessions = get_files(bd, '*file_list_for_batch_processing.pkl')
for f in tqdm(finished_sessions):
    file_dict = load_obj(f)
    if np.all(file_dict['is_processed'] == True):
        print(f)
        h5_file_list_to_combine = sort(get_files(os.path.dirname(f), '*_final_to_combine_*'))
        print(len(h5_file_list_to_combine))
        if len(h5_file_list_to_combine) > 0:
            cnt1 += 1
        else:
            cnt2 += 1

"""
checks 
same ending file name number 
same number of frames 

INSERT SAMSONS PREDICTIONS INTO THE FINAL H5 FILES 
"""
bd = 'H:\My Drive\WhACC_PROCESSING_FOLDER'
sess_list = utils.sort(utils.lister_it(utils.get_files(bd, '*_final_combined.h5'), remove_string='\\processing'))
save_key = 'contacts_samson_curated_for_transfer_learning_220707'
for h5 in tqdm(sess_list):
    sess_string = os.path.basename(os.path.dirname(utils.norm_path(h5, '/')))
    d = None
    samsons_contacts_list = []
    if sess_string in final_dict.keys():
        d = final_dict[sess_string]
        file_list_2 = [os.path.basename(k) for k in d['file_list']]

        with h5py.File(h5, 'r+') as h:
            labels = h['labels'][:]
            frame_nums_1 = h['frame_nums'][:]
            full_file_names_1 = h['full_file_names'][:]
            file_list_1 = [k.decode("ascii", "ignore") for k in full_file_names_1]
            for iii, k in enumerate(file_list_1):
                fn_1 = int(frame_nums_1[iii])
                vid_ind_to_contacts_dict = np.where(k == np.asarray(file_list_2))[0]

                if vid_ind_to_contacts_dict.shape[0] == 1:
                    assert fn_1 == d['frame_nums'][vid_ind_to_contacts_dict[0]]
                    contacts_tmp = d['contacts'][vid_ind_to_contacts_dict[0]]
                else:
                    #                     print(' no file overlap for this file --> ' + k)
                    contacts_tmp = np.ones(fn_1) * -1
                contacts_tmp = contacts_tmp.astype(np.float32)
                samsons_contacts_list.append(contacts_tmp)
        assert len(np.concatenate(samsons_contacts_list)) == len(labels), 'mismatch size'
#         utils.overwrite_h5_key(h5, save_key, np.concatenate(samsons_contacts_list))


"""
####################################################################################################

####################################################################################################
CREATING THE INDS TO THE SPLIT DATA FOR DIFFERENT BORDERS 
####################################################################################################
"""



split_ratio = np.asarray([6, 2, 2])
split_ratio = np.round(100*(split_ratio/np.sum(split_ratio))).astype(int)
tvt_name_string = ['train', 'val', 'test']

border_contact_lists = [[],[],[]]
for border in [3, 10, 20, 40, 80]:
    for h5 in tqdm(sess_list):
        with h5py.File(h5, 'r') as h:
            fn = h['frame_nums'][:]
            c = h['contacts_samson_curated_for_transfer_learning_220707'][:]
        good_trials = []
        for i, (f1, f2) in enumerate(utils.loop_segments(fn)):
            good_trials.append(not np.any(c[f1:f2]==-1))
        np.random.seed(42)
        good_trial_inds = np.where(good_trials)[0]
        T_V_Test_inds = utils.split_list(good_trial_inds, split_ratio)
        T_V_Test_inds = [sorted(k) for k in T_V_Test_inds]
        border_contact_lists = [[],[],[]]
        for i, (f1, f2) in enumerate(utils.loop_segments(fn)):
            c2 = c[f1:f2] # make new contacts ans set other trials to -1
            for ii, tmp1 in enumerate(T_V_Test_inds):
                if i in tmp1:
                    border_contact_lists[ii].append(c2)
                else:
                    border_contact_lists[ii].append(np.ones_like(c2)*-1)

#         border_inds_TVT = [[],[],[]]
        for i, TVT_inds in enumerate(border_contact_lists):
            border_inds = []
            for k in TVT_inds:
                x = utils.smooth(k, border)
                border_inds.append(x>0)
            final_save_inds = np.concatenate(border_inds)
            assert len(c) == len(final_save_inds), 'mis match size'
            save_name = str(border)+'_border_' + tvt_name_string[i] + '_set_' + str(split_ratio[i])+'%_data'
            utils.overwrite_h5_key(h5, save_name, final_save_inds)
"""
####################################################################################################
MAKE A FINAL SUBSET OF NUMPY ARRAYS 
####################################################################################################
"""

def foo_save(name_in, data):
    tmp1 = os.path.dirname(name_in)
    Path(tmp1).mkdir(parents=True, exist_ok=True)
    np.save(name_in, data)
bd = r'Q:\SAMSONS_DATA\\'
tmp_key = '80_border_test_set_20%_data'
utils.print_h5_keys(sess_list[0], 1, 0)
selected_keys = utils.lister_it(utils.print_h5_keys(sess_list[0], 1, 0), '80_border')

for tmp_key in selected_keys:
    tvt_name = tmp_key.split('_set_')[0].split('_')[-1]
    save_dir = bd + tmp_key.split('border')[0]+'border' + os.sep
    for h5 in tqdm(sess_list):
        add_name = os.path.basename(h5).split('_final_combined.h5')[0] + '_'
        with h5py.File(h5, 'r') as h:
            inds = h[tmp_key][:]

    #         print(sum(inds))
            final_2105 = h['final_features_2105'][inds, :]
            labels = h['contacts_samson_curated_for_transfer_learning_220707'][inds]
            assert final_2105.shape[0] == len(labels), 'labels and features dont match'
            name_in = save_dir+'/'+tvt_name + '/'+add_name
            foo_save(name_in+'final_2105', final_2105)
            foo_save(name_in+'labels', labels)


"""
####################################################################################################
LOAD THE DATA FOR MODELING 
####################################################################################################
"""

bd = r'Q:\SAMSONS_DATA\40_border\\'
tvt_x = [[],[],[]]
tvt_y = [[],[],[]]
for ii, k in enumerate(['train', 'val', 'test']):
    bd2 = bd+k
    final_2105 = utils.sort(utils.get_files(bd2, '*final_2105.npy'))
    labels = utils.sort(utils.get_files(bd2, '*labels.npy'))
    for i, (f1, f2) in enumerate(zip(final_2105, labels)):
        assert os.path.basename(f1)[:-15]==os.path.basename(f2)[:-11], 'files do not match'
        tvt_x[ii].append(np.load(f1, allow_pickle=True))
        tvt_y[ii].append(np.load(f2, allow_pickle=True))
    tvt_x[ii] = np.concatenate(tvt_x[ii])
    tvt_y[ii] = np.concatenate(tvt_y[ii])



"""
####################################################################################################
MAKE INDS FOR FULL TRIALS 
####################################################################################################
"""

bd = '/Volumes/GoogleDrive-114825029448473821206/.shortcut-targets-by-id/1pUteMEgx37rAyqujJWxVN9-ywFAqaxhF/WhACC_PROCESSING_FOLDER/'
files = utils.lister_it(utils.get_files(bd, '*final_combined.h5'), remove_string='processing_')

for h5 in tqdm(files):
    test_inds = image_tools.get_h5_key_and_concatenate(h5, '80_border_test_set_20%_data')
    frame_nums = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
    full_trial_test_inds = np.zeros_like(test_inds)
    for k1, k2 in utils.loop_segments(frame_nums):
        if np.any(test_inds[k1:k2]):
            full_trial_test_inds[k1:k2] = 1
    utils.overwrite_h5_key(h5, 'full_trial_test_inds', full_trial_test_inds)




"""

"""

def track(self, video_file, match_method='cv2.TM_CCOEFF'):
        """this function scans a template image across each frame of the video to identify the pole location.
        This assumes there is a pole at each frame. Cropping optimizes scanning by ~80% and uses the first frame
        as a point of reference.

        Parameters
        ----------
        video_file :

        match_method :
             (Default value = 'cv2.TM_CCOEFF')

        Returns
        -------

        """

        # width and height of img_stacks will be that of template (61x61)
        w, h = self.template_image.shape[::-1]
        max_match_val = []
        # open video at directory
        video = cv2.VideoCapture(video_file)
        if (video.isOpened() == False):
            print('error opening video file')
        frame_numbers = int(video.get(7))

        img_list = []
        loc_list = []
        # video.set(cv2.CAP_PROP_POS_FRAMES, 0)
        # success, og_frame = video.read()
        method = eval(match_method)
        crop_top_left = 0
        pole_center = 0
        for fn in range(frame_numbers):
            # iterate to next frame and crop using current details
            video.set(cv2.CAP_PROP_POS_FRAMES, fn)
            success, og_frame = video.read()

            # preprocess image
            if 'frame' in locals() and self.use_narrow_search_to_speed_up:
                frame, crop_top_left, crop_bottom_right = self.crop_image_from_top_left(og_frame,
                                                                                        crop_top_left2,
                                                                                        [w, h],
                                                                                        3)
            else:
                frame = og_frame
            img = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY).astype('uint8').copy()

            # Apply template Matching
            res = cv2.matchTemplate(img, self.template_image, method)
            min_val, max_val, min_loc, top_left = cv2.minMaxLoc(res)
            max_match_val.append(max_val)
            top_left = np.flip(np.asarray(top_left))

            # crop image and store
            crop_img, crop_top_left2, crop_bottom_right2 = self.crop_image_from_top_left(og_frame,
                                                                                         top_left + crop_top_left,
                                                                                         [w, h])
            img_list.append(crop_img)
            loc_list.append(np.flip(crop_top_left2))



        img_stack = np.array(img_list, dtype=np.uint8)
        loc_stack = np.array(loc_list)
        return img_stack, loc_stack, max_match_val


def crop_image_from_top_left(im, crop_top_left, size_crop, inflation=1):
        """This is an accessory function to track to improve tracking speed. This crops the initial large image into a smaller one, based on the inflation rate.
        Inflation rate of 3 = 3 x 3 template image size around the first guessed pole location.

        Parameters
        ----------
        im :

        crop_top_left :

        size_crop :

        inflation :
             (Default value = 3)

        Returns
        -------

        """
        inflation_shift = np.floor((np.asarray(size_crop) * (inflation - 1)) / 2).astype(int)

        size_crop = np.asarray(size_crop) * inflation
        imshape = np.asarray(im.shape[:2])

        # adjust for inflation and make copies of origianl crop inds
        crop_top_left = crop_top_left - inflation_shift
        crop_top_left2 = crop_top_left.copy()
        crop_bottom_right = crop_top_left + np.asarray(size_crop)
        crop_bottom_right2 = crop_bottom_right.copy()

        # adjust crop in case of out of bounds
        crop_top_left[crop_top_left < 0] = 0
        crop_bottom_right[crop_bottom_right > imshape] = np.asarray(imshape)[crop_bottom_right > imshape]

        # for placing cropped image in random noise image tensor (we do this in case of out of bounds issues all outputs are the same size)
        TL_adj = crop_top_left - crop_top_left2

        c_cropped = im[crop_top_left[0]:crop_bottom_right[0], crop_top_left[1]:crop_bottom_right[1], ...].astype(
            'uint8')

        # make random noise image tensor
        crop_dim = list(c_cropped.shape)
        crop_dim[:2] = size_crop.tolist()
        cropped_image = np.random.randint(256, size=crop_dim).astype('uint8')

        if len(crop_dim) > 2:  # crop 3D or more
            cropped_image[TL_adj[0]:TL_adj[0] + c_cropped.shape[0], TL_adj[1]:TL_adj[1] + c_cropped.shape[1],
            ...] = c_cropped
        else:  # crop 2D
            cropped_image[TL_adj[0]:TL_adj[0] + c_cropped.shape[0],
            TL_adj[1]:TL_adj[1] + c_cropped.shape[1]] = c_cropped

        return cropped_image, crop_top_left2, crop_bottom_right2

import cv2
from whacc.utils import h5_key_exists
def del_h5_key(h5_in, key_in):
    if h5_key_exists(h5_in, key_in):
        with h5py.File(h5_in, 'r+') as h:
            del h[key_in]
utils.del_h5_key = del_h5_key


mp4_bd = '/Users/phil/Desktop/samsons_eval/'
h5 = '/Users/phil/Desktop/samsons_eval/AH1179X24052021x4_final_to_combine_1_to_5_of_310.h5'
def insert_images_into_feature_data_h5(h5, mp4_bd, image_key = 'images'):
    frame_nums = image_tools.get_h5_key_and_concatenate(h5, 'frame_nums')
    # max_val_stack = image_tools.get_h5_key_and_concatenate(h5, 'max_val_stack')
    locations_x_y = image_tools.get_h5_key_and_concatenate(h5, 'locations_x_y')
    template_img = image_tools.get_h5_key_and_concatenate(h5, 'template_img')
    img_size = list(template_img.shape[:2])
    mp4_list = [mp4_bd+os.sep+os.path.basename(k.decode("ascii", "ignore")) for k in image_tools.get_h5_key_and_concatenate(h5, 'full_file_names')]

    h5_creator = image_tools.h5_iterative_creator(h5,
                                                 overwrite_if_file_exists=False,
                                                 max_img_height=img_size[0],
                                                 max_img_width=img_size[1],
                                                 close_and_open_on_each_iteration=True,
                                                 color_channel=True,
                                                 add_to_existing_H5=True,
                                                 ignore_image_range_warning=False,
                                                 dtype_img=h5py.h5t.STD_U8BE,
                                                 dtype_labels=h5py.h5t.STD_I32LE,
                                                 image_key_name = image_key,
                                                 label_key_name = 'TRASH')

    utils.del_h5_key(h5, h5_creator.img_key)
    utils.del_h5_key(h5, h5_creator.label_key_name)
    utils.del_h5_key(h5, 'multiplier')

    for i, (k1, k2) in enumerate(tqdm(utils.loop_segments(frame_nums), total=len(frame_nums))):
        video_file = mp4_list[i]
        video = cv2.VideoCapture(video_file)
        tmp_location = locations_x_y[k1:k2]
        if (video.isOpened() == False):
            print('error opening video file')
        frame_numbers = int(video.get(7))
        images = []
        for fn in range(frame_numbers):
            video.set(cv2.CAP_PROP_POS_FRAMES, fn)
            success, og_frame = video.read()
            locations = tmp_location[fn]
            crop_img, crop_top_left2, crop_bottom_right2 = crop_image_from_top_left(og_frame, [locations[1], locations[0]], img_size)
            images.append(crop_img)
        images = np.asarray(images)
        h5_creator.add_to_h5(images, -1*np.ones(images.shape[0]))

mp4_bd = '/Users/phil/Desktop/samsons_eval/'
h5 = '/Users/phil/Desktop/samsons_eval/AH1179X24052021x4_final_to_combine_1_to_5_of_310.h5'
utils.insert_images_into_feature_data_h5(h5, mp4_bd, image_key = 'images')


mod = utils.load_obj('/Users/phil/Downloads/model_save.pkl') # replace with the model file full path
utils.foo_predict(mod, '/Users/phil/Desktop/samsons_eval/')


from whacc import image_tools, utils
from whacc.touch_curation_GUI import touch_gui
h5 = '/Users/phil/Desktop/samsons_eval/AH1179X24052021x4_final_to_combine_1_to_5_of_310.h5'
temp_yhat = image_tools.get_h5_key_and_concatenate(h5, 'temp_yhat')
temp_labels_smoothed = 1*(utils.smooth(temp_yhat, 5)>0.5)
utils.overwrite_h5_key(h5, 'temp_labels_smoothed', temp_labels_smoothed)
touch_gui(h5, 'temp_yhat', label_write_key=None)






maxshape = list(images.shape)
maxshape[0] = None
maxshape

hf_file.create_dataset('images',
                        np.shape(images),
                        h5py.h5t.STD_U8BE,
                        # jk need this to not explode the size of the data... commented this out because I wanted to use not 0-255 numbers
                        maxshape=maxshape,
                        chunks=True,
                        data=images)




    video.set(cv2.CAP_PROP_POS_FRAMES, 1000)
    success, og_frame = video.read()
    plt.figure()
    plt.imshow(og_frame)

    tmp1 = locations_x_y[i*3000+1000]

    plt.plot(tmp1[0], tmp1[1], '+')

    crop_img, crop_top_left2, crop_bottom_right2 = crop_image_from_top_left(og_frame, [tmp1[1], tmp1[0]], [71,71])

    asdfasdf








for fn in range(frame_numbers):
    video.set(cv2.CAP_PROP_POS_FRAMES, fn)
    success, og_frame = video.read()


    crop_img, crop_top_left2, crop_bottom_right2 = self.crop_image_from_top_left(og_frame,
                                                                             top_left + crop_top_left,
                                                                             [w, h])


crop_img, crop_top_left2, crop_bottom_right2 = crop_image_from_top_left(og_frame, locations_x_y[0]-30, [71,71])

plt.imshow(crop_img)
plt.imshow(og_frame)
locations_x_y[0]

np.unique(locations_x_y, axis=0)



h5_dir= '\WhACC_PROCESSING_FOLDER\\' # replace this with a directory to any of the h5 files
##############################
utils.auto_combine_final_h5s(h5_dir, delete_extra_files=True)
##############################
mod = utils.load_obj(r'X:\PHILLIP\Samsons_models\model_saves_2\model_save.pkl') # replace with the model file full path
h5_files = utils.get_files(h5_dir, '*final_combined.h5') # list of your completed H5 data files
label_key = 'samsons_next_top_model' # key to find your raw predictions later
for h5 in h5_files:
    if not utils.h5_key_exists(h5, label_key) # if the key exists, skip it
        fd = image_tools.get_h5_key_and_concatenate(h5, 'final_features_2105') # get the x feature data
        yhat = mod.predict(fd) # predictions
        utils.overwrite_h5_key(h5, label_key, yhat) # write to H5 file with new name



import copy
import matplotlib.pyplot as plt

import numpy as np
from scipy.signal import medfilt, medfilt2d

from whacc import image_tools
from whacc import utils
import pandas as pd
import seaborn as sns
from matplotlib.lines import Line2D

def foo_rename(instr):
    if isinstance(instr, list):
        for i, k in enumerate(instr):
            instr[i] = foo_rename(k)
        return instr
    else:
        return '/Volumes/GoogleDrive-114825029448473821206/My Drive' + instr.split('My Drive')[-1]


def medfilt_confidence_scores(pred_bool_in, kernel_size_in):
    if len(pred_bool_in.shape) == 1 or pred_bool_in.shape[1] == 1:
        pred_bool_out = medfilt(copy.deepcopy(pred_bool_in).flatten(), kernel_size=kernel_size_in)
    else:
        pred_bool_out = medfilt2d(copy.deepcopy(pred_bool_in), kernel_size=[kernel_size_in, 1])
    return pred_bool_out


def confidence_score_to_class(pred_bool_in, thresh_in):
    if len(pred_bool_in.shape) == 1 or pred_bool_in.shape[1] == 1:
        pred_bool_out = ((pred_bool_in > thresh_in) * 1).flatten()
    else:
        pred_bool_out = np.argmax(pred_bool_in, axis=1)
    #     NOTE: threshold is not used for the multi class models
    return pred_bool_out


def foo_arg_max_and_smooth(pred_bool_in, kernel_size_in, thresh_in, key_name_in, L_key_=None, L_type_split_ind=None):
    pred_bool_out = medfilt_confidence_scores(pred_bool_in, kernel_size_in)
    pred_bool_out = confidence_score_to_class(pred_bool_out, thresh_in)
    if L_key_ is None:
        L_key_ = '_'.join(key_name_in.split('__')[L_type_split_ind].split(' '))

    pred_bool_out = utils.convert_labels_back_to_binary(pred_bool_out, L_key_)
    return pred_bool_out


"""##################################################################################################################"""
"""##################################################################################################################"""
# key_name = 'model_5____3lag__regular_labels__MODEL_FULL_TRAIN_rollSTD_3_7_rollMEAN_3_7_shift_neg2_to_2_val_with_real_val_PM_JC_LIGHT_GBM'
# h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/colab_data2/model_testing/all_data/final_predictions/ALT_LABELS_FINAL_PRED/AH0407_160613_JC1003_AAAC_ALT_LABELS.h5'

"""
pred key 
TEMP_yhat_full_192000_nan

frame nums
fn_192000

~~other important keys~~ 
full_test_labels
no_hair_inds
in_range_by_hand_192000 ~~ has only 0's and 1's
pole_in_range ~~ has -1's 0's and 1's 
"""
# h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/FINAL_TEST_SPLIT_BY_TRIAL_FULL_TRIALS.h5'
# d = utils.h5_to_dict(h5_file)
# utils.info(d)
# utils.get_dict_info(d)


h5_file = '/Volumes/GoogleDrive-114825029448473821206/My Drive/LIGHT_GBM/H5_data/3lag_numpy_aug_for_final_LGBM/keep_inds_no_pole_hair_no_skip_frames_for_SPLIT_BY_TRIAL/FINAL_TEST_SPLIT_BY_TRIAL_FULL_TRIALS.h5'
# utils.print_h5_keys(h5_file)
# tmp1 = utils.getkey(h5_file, 'AH0407_160613_JC1003_AAAC_3lag.npy')

key_name = 'TEMP_yhat_full_192000_nan' # predictions
L_key_ = '[0, 1]- (no touch, touch)' # format of the predictions, wont change anything in this case but still needed
real_bool_key = 'full_test_labels'
kernel_size = 5
threshold = 0.5


# np.sum(utils.getkey(h5_file, 'pole_in_range')==1) # 14036
# np.sum(utils.getkey(h5_file, 'in_range_by_hand_192000')==1) # 114051
# np.sum(utils.getkey(h5_file, 'no_hair_inds')==1) # 174000

# L_key_ = None
fn_key = 'full_test_fn' #sum to 174000 becasue it removes one trial, use no_hair_inds on 192000 data to match it
in_range_key = 'in_range_by_hand_192000'
# in_range_key = 'in_range_by_hand_192000' # pole_in_range?????
#
# # tmp1 = utils.getkey(h5_file, 'pole_in_range')
# # plt.plot(tmp1)
"""##################################################################################################################"""
"""##################################################################################################################"""
pred_bool_temp = image_tools.get_h5_key_and_concatenate(h5_file, key_name)
pred_bool_temp = pred_bool_temp.astype(float)
pred_bool_smoothed = foo_arg_max_and_smooth(pred_bool_temp, kernel_size, threshold, key_name, L_key_=L_key_)

real_bool = image_tools.get_h5_key_and_concatenate(h5_file, real_bool_key)
# trial_nums_and_frame_nums = image_tools.get_h5_key_and_concatenate(h5_file, 'trial_nums_and_frame_nums')
# frame_nums = trial_nums_and_frame_nums[1, :].astype(int)
frame_nums = utils.getkey(h5_file, fn_key)

in_range = image_tools.get_h5_key_and_concatenate(h5_file, in_range_key)
"""##################################################################################################################"""
"""##################################################################################################################"""

all_h5s = utils.get_h5s(foo_rename('/content/gdrive/My Drive/Colab data/curation_for_auto_curator/finished_contacts/'),
                        print_h5_list=False)
tmp_h_cont, h_names = utils._get_human_contacts_(all_h5s)
"""##################################################################################################################"""
"""####################### correct 192000 dat to 17400 ##############################################################"""

"""##################################################################################################################"""
"""##################################################################################################################"""
data_key_inds = utils.lister_it(utils.print_h5_keys(h5_file, 1, 0), '.npy')
h_cont = []
for hi, key_i in zip(tmp_h_cont, data_key_inds):
    inds = utils.getkey(h5_file, key_i)==1
    x = hi[:, inds]
    h_cont.append(x)
human = np.hstack(h_cont)

no_hair_inds = utils.getkey(h5_file, 'no_hair_inds')

keep_inds = np.logical_and(no_hair_inds==1, in_range==1)==True

whacc = pred_bool_smoothed[keep_inds.astype(bool)]
human = human[:, keep_inds.astype(bool)]

human_mean = np.mean(human, axis=0)




def foo_agree_0_1(in1, in2):
    # full0agree = np.mean(1 * (in1 == 0) == 1 * (in2 == 0))
    # full1agree = np.mean(1 * (in1 == 1) == 1 * (in2 == 1))
    full0agree = np.mean((in1 == 0) == (in2 == 0))
    full1agree = np.mean((in1 == 1) == (in2 == 1))
    return full0agree, full1agree


# np.mean(full0agree)*100, np.mean(full1agree)*100
d = {'h0': [], 'h1': [],'w0': [], 'w1': []}
for i, a in enumerate(human):
    inds = np.ones(human.shape[0]).astype(bool)
    inds[i] = False
    tmp_mean = np.mean(human[inds], axis=0) # index out 2 human curator contacts
    full0agree, full1agree = foo_agree_0_1(a, tmp_mean) #comapre human to other 2 humans
    d['h0'].append(full0agree)
    d['h1'].append(full1agree)
    full0agree, full1agree = foo_agree_0_1(whacc, tmp_mean) #compare whac to other 2 humans
    d['w0'].append(full0agree)
    d['w1'].append(full1agree)

# df = pd.DataFrame(d)
# ax = sns.pointplot(x="h0", y="w0", data=df)
# ax = sns.pointplot(x="h1", y="w1", data=df)
#
# df = pd.DataFrame(d)
# ax = sns.scatterplot(x="h0", y="h1", data=df)
# ax = sns.scatterplot(x="w0", y="w1", data=df)
# plt.xlim([0.9930, .997])
# plt.ylim([0.9930, .997])


# np.mean(full0agree)*100, np.mean(full1agree)*100
d = {'percent correct':[], 'ind':[], 'label':[], 'class':[]}
for i, a in enumerate(human):
    inds = np.ones(human.shape[0]).astype(bool)
    inds[i] = False
    tmp_mean = np.mean(human[inds], axis=0)
    d['percent correct']+=list(foo_agree_0_1(a, tmp_mean))
    d['percent correct']+=list(foo_agree_0_1(whacc, tmp_mean))
    d['ind']+=[i]*4
    d['label']+=['human', 'human', 'WhACC', 'WhACC']
    d['class']+=['no touch', 'touch']*2

df = pd.DataFrame(d)
plt.figure(figsize=(7, 6))
ax = sns.pointplot(x="label", y="percent correct", hue="class", data=df, dodge=True)

plt.figure(figsize=(7, 6))
ax = sns.pointplot(x="label", y="percent correct", data=df, dodge=True)

plt.figure(figsize=(7, 6))
for k in range(3):
    ax = sns.pointplot(x="label", y="percent correct", data=df.loc[df['ind'] == k], dodge=True, ci=None)




def foo_agree_all(in1, in2):
    return np.mean(np.append([1 * (in1 == 0) == 1 * (in2 == 0)], [1 * (in1 == 1) == 1 * (in2 == 1)]))

d = {'percent correct':[], 'ind':[], 'label':[], 'class':[]}
for i, a in enumerate(human):
    inds = np.ones(human.shape[0]).astype(bool)
    inds[i] = False
    tmp_mean = np.mean(human[inds], axis=0)
    d['percent correct'].append(foo_agree_all(a, tmp_mean))
    d['percent correct'].append(foo_agree_all(whacc, tmp_mean))
    d['ind']+=[i]*2
    d['label']+=['human', 'WhACC']
    d['class']+=['all']*2


df = pd.DataFrame(d)
plt.figure(figsize=(7, 6))
ax = sns.pointplot(x="label", y="percent correct", data=df, dodge=True, ci = False, color='k', legend='sadf')

for k in range(3):
    ax = sns.pointplot(x="label", y="percent correct", data=df.loc[df['ind'] == k], dodge=True, ci=None, color='b')

custom_lines = [Line2D([0], [0], color='k', lw=4),
                Line2D([0], [0], color='b', lw=4)]
plt.legend(custom_lines, ['mean', 'individual'])



#######################################################################################################################
#######################################################################################################################
'''evaluate the total agree percentage of the touch count'''
#######################################################################################################################
#######################################################################################################################
#######################################################################################################################
d = dict()

""" touch count error version of the plot"""
from whacc import analysis
frame = frame_nums
human_performance = []
whacc_performance = []

human_temp = copy.deepcopy(list(human))

all_agree_inds = np.all(human[0] == human, axis = 0)
print('percent full agree ', np.mean(all_agree_inds))
d['percent humans full agree'] = np.mean(all_agree_inds)
d['humans full agree inds'] = all_agree_inds

human_temp2 = [k[all_agree_inds] for k in human_temp]
whacc2 = copy.deepcopy(whacc)[all_agree_inds]


d['percent whacc agree with human full agree'] = np.mean(whacc2==human_temp2[0])

utils.info(d)

#######################################################################################################################
#######################################################################################################################
#######################################################################################################################
# # d = {'x' : [], 'and_or_labels':['human AND', 'human OR', 'WhACC AND', 'WhACC OR'],
# #                      'error_labels': ['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct']}
# d['error_type'] = []
# d['x'] = []
# for k in ['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct', 'pair_ind', 'human_or_whacc', 'and_or']:
#     d[k] = []
# """ touch count error version of the plot"""
# from whacc import analysis
# # frame = frame_nums
# human_performance = []
# whacc_performance = []
# for ind in range(3):
#     human_two = copy.deepcopy(list(human))
#     human_one = np.asarray(human_two.pop(ind))
#     human_two = np.asarray(human_two)
#
#     x_and = np.mean(human_two, axis = 0)
#     x_and[x_and<1] = 0
#
#     x_or = np.mean(human_two, axis = 0)
#     x_or[x_or>0] = 1
#
#     df_list = []
#     df_list.append(
#         analysis.thresholded_error_types(x_and, human_one, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))
#     df_list.append(
#         analysis.thresholded_error_types(x_or, human_one, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))
#     df_list.append(
#         analysis.thresholded_error_types(x_and, whacc, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))
#     df_list.append(
#         analysis.thresholded_error_types(x_or, whacc, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))
#
#     np.mean([df_list[0], df_list[1]], axis = 0)
#     np.mean([df_list[2], df_list[3]], axis = 0)
# # get TCerror for each and take the mean
#     plt.figure()
#     x = np.squeeze(np.asarray(df_list))
#     d['x'].append(x.T)
#     plt.plot(x.T, label = ['human AND', 'human OR', 'WhACC AND', 'WhACC OR'])
#     plt.xlabel(['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct'])
#     plt.legend()
#     print(np.max(np.max(x, axis = 0)[:-2]))
#     plt.ylim([0, 45])
#
# utils.info(d)
# type_compare = 'AND'
#
# and_or_inds = np.where([type_compare in k for k in d['and_or_labels']])[0]
# x2 = []
#
# for ii in range(3):
#     x = d['x'][ii][:, and_or_inds]
#     x2.append(x)
# for i, k in enumerate(d['error_labels']):
#     np.asarray(x2).T.shape # humans 1 to 3 , error type , human vs whacc



#######################################################################################################################
#######################################################################################################################
###########################$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$##########################################################

###########################$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$##########################################################

# d = {'x' : [], 'and_or_labels':['human AND', 'human OR', 'WhACC AND', 'WhACC OR'],
#                      'error_labels': ['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct']}
d = dict()
for k in ['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct', 'pair_ind', 'human_or_whacc', 'and_or']:
    d[k] = []
""" touch count error version of the plot"""
from whacc import analysis
# frame = frame_nums
human_performance = []
whacc_performance = []
for ind in range(3):
    human_two = copy.deepcopy(list(human))
    human_one = np.asarray(human_two.pop(ind))
    human_two = np.asarray(human_two)
    real_count = np.sum(np.diff(1*(np.mean(human, axis = 0)>.5))==1)
    x_and = np.mean(human_two, axis = 0)
    x_and[x_and<1] = 0
    and_count = np.sum(np.diff(x_and)==1)
    x_or = np.mean(human_two, axis = 0)
    x_or[x_or>0] = 1
    or_count = np.sum(np.diff(x_or)==1)
    print(real_count, and_count, or_count)

    df_list = []
    df_list.append(
        analysis.thresholded_error_types(x_and, human_one, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))
    df_list.append(
        analysis.thresholded_error_types(x_or, human_one, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))
    df_list.append(
        analysis.thresholded_error_types(x_and, whacc, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))
    df_list.append(
        analysis.thresholded_error_types(x_or, whacc, edge_threshold=4, frame_num_array=frame_nums, thresholds=[0.5]))

    # np.mean([df_list[0], df_list[1]], axis = 0)
    # np.mean([df_list[2], df_list[3]], axis = 0)
    x = np.squeeze(np.asarray(df_list))
    x2 = []
    for kk, div_by in zip(x, [and_count, or_count, and_count, or_count]):
        x2.append(kk/div_by)
    x = np.asarray(x2)
    for ii, e_type in enumerate(['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct']):
        d[e_type].append(x[:, ii])
    d['human_or_whacc'].append(['human', 'human', 'WhACC', 'WhACC'])
    d['and_or'].append(['and', 'or', 'and', 'or'])
    d['pair_ind'].append([ind]*4)


for k in ['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct', 'pair_ind', 'human_or_whacc', 'and_or']:
    d[k] = np.concatenate(d[k])
df2 = pd.DataFrame(d)


plt.figure(figsize=(7, 6))
# ax = sns.pointplot(x="label", y="ghost", hue="class", data=df2, dodge=True)
ax = sns.pointplot(x="human_or_whacc", y="ghost", data=df2, dodge=True)

ax = sns.pointplot(x="human_or_whacc", y="ghost", data=df2, dodge=True)



plt.figure(figsize=(7, 6))
for k in range(3):
    x_in = df2.loc[np.logical_and(df2['pair_ind'] == k, df2['and_or'] == 'and')]
    ax = sns.pointplot(x='human_or_whacc', y='ghost', data=x_in, dodge=True, color = None, ci=None)


######
from matplotlib.pyplot import cm
n = 8
color = cm.tab10(np.linspace(0, 1, n))
# color = cm.jet(np.linspace(0, 1, n))

for i, c in zip(range(n), color):
   plt.scatter(i, i, c=c)

####

tmp1 = df2.iloc[:, :8].unstack().reset_index()
tmp2 = np.tile(np.asarray(df2.iloc[:, 8:]).T,8).T

for i, k in enumerate(['pair_ind', 'human_or_whacc', 'and_or']):
    tmp1[k] =tmp2[:, i]
tmp1['y'] = tmp1[0]
###########################$$$$$$$$$
plt.figure(figsize=(7, 6))
alpha = 0.3
for k in range(3):
    x_in = tmp1.loc[np.logical_and(tmp1['pair_ind'] == k, tmp1['and_or'] == 'and')]
    ax = sns.pointplot(x='human_or_whacc', y='y', data=x_in,  edgecolor=None, linewidth=0, markers='o', hue='level_0', dodge=True, palette=color, linestyles='', plot_kws=dict(alpha=alpha)) # plot_kws={'alpha':0.1}
plt.setp(ax.collections, alpha=alpha) #for the markers
plt.setp(ax.lines, alpha=alpha)       #for the lines
plt.legend('')


###
# ax = sns.scatterplot(x='human_or_whacc', y='y', data=x_in, hue='level_0', palette=color, dodge=True)
# plt.setp(ax.collections, alpha=alpha) #for the markers
# plt.setp(ax.lines, alpha=alpha)       #for the lines
# plt.legend('')
###########################$$$$$$$$$
# color[:, -1] = 1
hue_order = ['split', 'ghost', 'miss', 'join', 'deduct', 'append', 'deduct++', 'append++']
hue_order = ['split', 'ghost', 'miss', 'join', 'deduct++', 'append++', 'deduct', 'append']


x_in = tmp1.loc[tmp1['and_or'] == 'and']
ax = sns.pointplot(x='human_or_whacc', y='y', data=x_in, hue='level_0',hue_order=hue_order, dodge=False, ci=.95, palette=color)
plt.ylabel('Errors per touch'); plt.xlabel('');plt.ylim([0, .1])
###########################$$$$$$$$$

x2 = x_in.groupby(['human_or_whacc'])
sub_data = x2.get_group('WhACC').groupby('level_0').mean()['y'] - x2.get_group('human').groupby('level_0').mean()['y']

sub_data = pd.DataFrame(sub_data).reset_index()
# sub_data['index'] = [7, 4, 6, 5, 2, 0, 3, 1]
# sub_data = sub_data.set_index('index').sort()
#
# sub_data.sort_values(['index'])
sns.barplot(x = 'level_0', y = 'y', data=sub_data, palette=color, order=hue_order)
# plt.hlines(0, 0, 8, 'k')
plt.grid('on')
plt.ylabel('WhACC - Human (error rate)'); plt.xlabel('');

# plt.ylim([-.035, .035])

###########################$$$$$$$$$


# sub_data2 = np.asarray(x2.get_group('WhACC')['y']) - np.asarray(x2.get_group('human')['y'])
# plt.bar(range(len(sub_data2)), sub_data2)





x2 = x_in.groupby(['level_0', 'human_or_whacc'])
# x2.get_group()
x2.head(100)
x2.mean(['level_0'])

###########################$$$$$$$$$

###########################$$$$$$$$$


x_in = tmp1.loc[tmp1['and_or'] == 'and']
x_in = x_in.iloc[:36] # only TC errors

tmp3 = x_in.groupby(['pair_ind', 'human_or_whacc', 'level_0'])
tmp4 = tmp3['y'].mean()

tmp5 = tmp4.groupby(['pair_ind', 'human_or_whacc'])
tmp6 = tmp5.sum().reset_index()
tmp6 = pd.DataFrame(tmp6)
###########################$$$$$$$$$
ax = sns.pointplot(x='human_or_whacc', y='y', data=tmp6, dodge=True, ci=.95)
###########################$$$$$$$$$
for k in range(3):
    x_in = tmp6.loc[tmp6['pair_ind'] == k]
    ax = sns.pointplot(x='human_or_whacc', y='y', data=x_in, dodge=True, ci=None)

# ax = sns.pointplot(x='human_or_whacc', y='y', data=x_in, hue='level_0', dodge=True, ci=None)
###########################$$$$$$$$$

for k in range(3):
    x_in = tmp1.loc[np.logical_and(tmp1['pair_ind'] == k, tmp1['and_or'] == 'and')]
    ax = sns.pointplot(x='human_or_whacc', y='y', data=x_in, hue='level_0', dodge=True, ci=None)


ax = sns.pointplot(x='human_or_whacc', y='y', data=x_in, dodge=True)





tmp1 = df2.unstack().reset_index()
tmp1['data'] = tmp1[0]
ax = sns.pointplot(y='data', hue="level_0", data=tmp1, dodge=True)


###########################$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$##########################################################

###########################$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$##########################################################

#######################################################################################################################
#######################################################################################################################
#######################################################################################################################

""" touch count error version of the plot but with consensus rules and disagree touches are removed"""
from whacc import analysis

human_performance = []
whacc_performance = []
for ind in range(3):
    human_two = copy.deepcopy(list(human))
    human_one = np.asarray(human_two.pop(ind))
    human_two = list(np.asarray(human_two))

    inds = human_two[0] == human_two[1]

    human_two[0] = np.asarray(human_two[0])[inds]
    human_two[1] = np.asarray(human_two[1])[inds]
    human_one = human_one[inds]
    whacc_tmp = whacc[inds]


    x_conc = np.mean(human_two, axis = 0)
    x_conc[x_conc<1] = 0

    # x_or = np.mean(human_two, axis = 0)
    # x_or[x_or>0] = 1

    df_list = []
    df_list.append(
        analysis.thresholded_error_types(x_conc, human_one, edge_threshold=4, frame_num_array=frame, thresholds=[0.5]))
    # df_list.append(analysis.thresholded_error_types(x_or, human_one, edge_threshold=4, thresholds=[0.5]))
    df_list.append(
        analysis.thresholded_error_types(x_conc, whacc_tmp, edge_threshold=4, frame_num_array=frame, thresholds=[0.5]))
    # df_list.append(analysis.thresholded_error_types(x_or, whacc, edge_threshold=4, thresholds=[0.5]))

    np.mean([df_list[0], df_list[1]], axis = 0)
    # np.mean([df_list[2], df_list[3]], axis = 0)

# get TCerror for each and take the mean
    plt.figure()
    x = np.squeeze(np.asarray(df_list))
    plt.plot(x.T, label = ['human consensus', 'WhACC consensus'])
    plt.xlabel(['ghost', 'miss', 'join', 'split', 'append++', 'deduct++', 'append', 'deduct'])
    plt.legend()
    print(np.max(np.max(x, axis = 0)[:-2]))
    plt.ylim([0, 45])





from whacc import utils
from whacc import image_tools

bd = '/Users/phil/Dropbox/Autocurator/data/samsons_subsets/use/train_and_validate/'
H5_list_to_train = utils.get_h5s(bd)
H5_list_to_train = utils.lister_it(H5_list_to_train,
                                   keep_strings=['subset'])  # get only the H5 files with the word 'subset'
print(H5_list_to_train)
split_h5_files = image_tools.split_h5(H5_list_to_train, [8, 3], temp_base_name=[bd + 'training_set', bd + 'validation_set'],
                                      add_numbers_to_name=False)
#_________
bd = '/Users/phil/Dropbox/Autocurator/data/samsons_subsets/use/test/'
H5_list_to_train = utils.get_h5s(bd)
H5_list_to_train = utils.lister_it(H5_list_to_train,
                                   keep_strings=['subset'])  # get only the H5 files with the word 'subset'
print(H5_list_to_train)
split_h5_files = image_tools.split_h5(H5_list_to_train, [1], temp_base_name=[bd + 'test_set'],
                                      add_numbers_to_name=False)

# import h5py
# import numpy as np
# H5_list_to_train = utils.get_h5s('/Users/phil/Dropbox/Autocurator/data/samsons_subsets/use/test/')
# for k in H5_list_to_train:
#     with h5py.File(k, 'r') as h:
#         print(k)
#         print(len(np.unique(h['labels'][:])))
#         # print(h.keys())
#
# #
# # H5_list_to_train = utils.get_h5s('/Users/phil/Dropbox/Autocurator/data/samsons_subsets/train_and_validate/')
# # H5_list_to_train = utils.lister_it(H5_list_to_train,
# #                                    keep_strings=['subset'])  # get only the H5 files with the word 'subset'
# # print(H5_list_to_train)
# # bd = '/Users/phil/Dropbox/Autocurator/data/samsons_subsets/train_and_validate/'  # base directory to put files
# # split_h5_files = image_tools.split_h5(H5_list_to_train, [8, 3], temp_base_name=[bd + 'training', bd + 'validation'],
# #                                       add_numbers_to_name=False)
